package pps.core.common.cache;

import xstartup.base.XContext;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XSingleCache;
import xstartup.core.base.annotation.XCacheKeyPrefix;

import java.util.function.Supplier;

@XCacheKeyPrefix("CONFIG_CACHE")
public class ConfigCache implements XCacheObject, XCacheLife {

    /*
     * 配置名称
     * */
    private String configName;
    /*
     * 配置值
     * */
    private String configValue;

    /**
     * 获取缓存的token
     */
    public static ConfigCache get(XContext context, String configName, boolean deleteOriginal, Supplier<String> valueProvider) {
        ConfigCacheRefreshTool tool = ConfigCacheRefreshTool.get(ConfigCacheRefreshTool.class);
        tool.setValueProvider(valueProvider);
        ConfigCache cache = tool.find(context, configName, ConfigCache.class);
        if (deleteOriginal) {
            try {
                tool.delete(context, cache);
            } catch (Exception e) {
                context.getLogger().error("delete cache exception.", e);
            }
            cache = tool.find(context, configName, ConfigCache.class);
        }

        return cache;
    }

    public String getConfigName() {
        return configName;
    }

    public void setConfigName(String configName) {
        this.configName = configName;
    }

    public String getConfigValue() {
        return configValue;
    }

    public void setConfigValue(String configValue) {
        this.configValue = configValue;
    }

    @Override
    public Integer getDuration() {
        //30秒
        return 30;
    }

    @Override
    public String getCacheKey() {
        return configName;
    }

    static class ConfigCacheRefreshTool extends XSingleCache<ConfigCache> {
        private Supplier<String> valueProvider;

        public Supplier<String> getValueProvider() {
            return valueProvider;
        }

        public void setValueProvider(Supplier<String> valueProvider) {
            this.valueProvider = valueProvider;
        }

        @Override
        protected ConfigCache restore(XContext context, String cacheKey) {
            //生成缓存对象
            if (valueProvider != null) {
                ConfigCache cache = new ConfigCache();
                cache.configName = cacheKey;
                cache.configValue = valueProvider.get();
                return cache;
            }
            return null;
        }
    }
}
