package pps.core.common.cache;

import lombok.Data;
import xstartup.base.XContext;
import xstartup.cache.XCacheLife;
import xstartup.cache.XCacheObject;
import xstartup.cache.XSingleCache;

/**
 * 定时任务Redis锁
 *
 * @author ZWT
 * @date 2023/12/21 14:36
 */
@Data
public class TaskLockCache implements XCacheObject, XCacheLife {
    /**
     * Redis Key
     */
    private String redisKey;
    /**
     * Redis Value
     */
    private String redisValue;

    /**
     * 检查缓存是否存在指定的code
     *
     * @param context  上下文
     * @param redisKey redis键
     * @return {@link TaskLockCache}
     */
    public static TaskLockCache exist(XContext context, String redisKey) {
        return XSingleCache.get(Tool.class).find(context, redisKey, TaskLockCache.class);
    }

    /**
     * 删去
     *
     * @param context  上下文
     * @param redisKey redis键
     */
    public static void delete(XContext context, String redisKey) {
        XSingleCache.get(Tool.class).delete(context, redisKey, TaskLockCache.class);
    }

    /**
     * 设置缓存
     *
     * @param context 上下文
     * @param cache   隐藏物
     */
    public static void set(XContext context, TaskLockCache cache) {
        XSingleCache.get(Tool.class).set(context, cache);
    }

    /**
     * 获取持续时间
     *
     * @return {@link Integer}
     */
    @Override
    public Integer getDuration() {
        return 600;
    }

    /**
     * 获取缓存密钥
     *
     * @return {@link String}
     */
    @Override
    public String getCacheKey() {
        return this.redisKey;
    }

    /**
     * 工具
     *
     * @author ZWT
     * @date 2023/12/21
     */
    static class Tool extends XSingleCache<TaskLockCache> {
        @Override
        protected TaskLockCache restore(XContext context, String cacheKey) {
            return null;
        }
    }
}
