package pps.core.common.utils;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

public class ManifestComponentInfoHelper {

    private final static HashMap<String, ManifestComponentInfo> manifestComponentInfoHashMap = new HashMap<>();
    private final static String COMPONENT_GROUP_NAME = "Component-Group-Name";
    private final static String COMPONENT_ARTIFACTID = "Component-Artifactid";
    private final static String COMPONENT_VERSION = "Component-Version";
    private final static String COMPONENT_BUILD_TIME = "Component-Build-Time";
    private final static String COMPONENT_DESCRIPTION = "Component-Description";

    public static void init(ClassLoader classLoader) {
        try {
            Enumeration<URL> resources = classLoader.getResources("META-INF/MANIFEST.MF");
            while (resources.hasMoreElements()) {
                try {
                    InputStream stream = resources.nextElement().openStream();

                    final HashMap<String, String> manifestProperties = new HashMap<>(10);
                    final Manifest manifest = new Manifest(stream);
                    Attributes attributes = manifest.getMainAttributes();
                    attributes.entrySet().stream().forEach(entry -> {
                        manifestProperties.put(entry.getKey().toString(), entry.getValue().toString());
                    });

                    if (manifestProperties.containsKey(COMPONENT_GROUP_NAME)
                            && manifestProperties.containsKey(COMPONENT_ARTIFACTID)) {
                        ManifestComponentInfo manifestComponentInfo = new ManifestComponentInfo();
                        manifestComponentInfo.setGroupName(manifestProperties.get(COMPONENT_GROUP_NAME));
                        manifestComponentInfo.setArtifactid(manifestProperties.get(COMPONENT_ARTIFACTID));
                        manifestComponentInfo.setVersion(manifestProperties.get(COMPONENT_VERSION));
                        manifestComponentInfo.setBuildTime(manifestProperties.get(COMPONENT_BUILD_TIME));
                        manifestComponentInfo.setDescription(manifestProperties.get(COMPONENT_DESCRIPTION));
                        String k = String.format("%s:%s", manifestComponentInfo.getGroupName(), manifestComponentInfo.getArtifactid());
                        manifestComponentInfoHashMap.put(k, manifestComponentInfo);
                    }

                    stream.close();
                } catch (IOException E) {
                    // handle
                }
            }

        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }

    public static ManifestComponentInfo getManifestComponentInfo(String groupName, String artifectId) {
        String k = String.format("%s:%s", groupName, artifectId);
        if (manifestComponentInfoHashMap.containsKey(k)) {
            return manifestComponentInfoHashMap.get(k);
        } else {
            ManifestComponentInfo manifestComponentInfo = new ManifestComponentInfo();
            manifestComponentInfo.setGroupName(groupName);
            manifestComponentInfo.setArtifactid(artifectId);
            manifestComponentInfo.setVersion("N/A");
            manifestComponentInfo.setBuildTime("N/A");
            manifestComponentInfo.setDescription("N/A");
            return manifestComponentInfo;
        }
    }
}
