package pps.core.common.utils;

import org.apache.commons.lang3.StringUtils;

import java.util.regex.Pattern;

/**
 * @author lixueyan
 * @date 2022/11/11 0011 10:31
 */
public class PatternUtil {
    private static final String phonePattern = "^(13[0-9]|14[01456879]|15[0-35-9]|16[2567]|17[0-8]|18[0-9]|19[0-35-9])\\d{8}$";

    public static boolean checkPhone(String phone) {
        //用户手机号码校验
        if (Pattern.matches(phonePattern, phone)) {
            return true;
        } else {
            return false;
        }

    }

    /**
     * 字符串脱敏模糊处理
     */
    public static String desensitization(String str) {
        if (StringUtils.isBlank(str)) {
            return str;
        }
        //先判断手机号
        String s = rePhone(str);
        if (!s.contains("****")) {
            //在判断身份证号
            s = reNo(str);
        }
        return s;
    }

    /**
     * 身份证号简单校验
     *
     * @param IdCard
     * @return
     */
    public static boolean easyCheckIdCard(String IdCard) {
        //字符串为空或者不符合简单的18位校验返回false
        String regex = "^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$";
        if (StringUtils.isBlank(IdCard) || !Pattern.matches(regex, IdCard)) {
            return false;
        }
        return true;
    }

    //身份证号加*号
    public static String reNo(String realNo) {
        String newNo = "";
        if (easyCheckIdCard(realNo)) {
            if (realNo.length() == 15)
                newNo = realNo.replaceAll("(\\d{4})\\d{7}(\\d{4})", "$1*******$2");
            if (realNo.length() == 18)
                newNo = realNo.replaceAll("(\\d{4})\\d{10}(\\d{4})", "$1**********$2");
        } else {
            newNo = reStr(realNo);
        }
        return newNo;
    }

    //邮箱加*号
    public static String reStr(String str) {
        if (StringUtils.isBlank(str)) {
            return str;
        }
        String name = str;
        if (str.contains("@") && str.length() > 9) {
            StringBuilder stringBuilder = new StringBuilder(str);
            name = stringBuilder.replace(3, 7, "****").toString();
        }
        return name;
    }

    //手机号加*号
    public static String rePhone(String realPhone) {
        String phoneNumber = "";
        if (realPhone.length() == 11 && checkPhone(realPhone))
            phoneNumber = realPhone.replaceAll("(\\d{3})\\d{4}(\\d{4})", "$1****$2");
        else
            phoneNumber = realPhone;
        return phoneNumber;
    }
}
