package pps.core.common.utils;

public enum WellKnownRegex {
    Email_ENG(1, "^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$"),
    Email_CHN(2, "^[A-Za-z0-9\\u4e00-\\u9fa5]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$"),
    //电话号码
    PHONE(3, "^1(3|4|5|6|7|8|9)\\d{9}$"),
    URL1(4, "^((http:\\/\\/)|(https:\\/\\/))?([a-zA-Z0-9]([a-zA-Z0-9\\-]{0,61}[a-zA-Z0-9])?\\.)+[a-zA-Z]{2,6}(\\/)"),
    URL2(5, "(https?:\\/\\/)?(www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{2,256}\\.[a-z]{2,6}\\b([-a-zA-Z0-9@:%_\\+.~#?&//=]*)"),
    IPV4(6, "((?:(?:25[0-5]|2[0-4]\\d|[01]?\\d?\\d)\\.){3}(?:25[0-5]|2[0-4]\\d|[01]?\\d?\\d))"),

    //英文字母和数字
    ENGLETTER_NUM(7, "^[A-Za-z0-9]+$"),
    ACCOUNTNAME(8, "^[a-zA-Z][a-zA-Z0-9_]{4,15}$"),
    //汉字
    CHINESE(9, "^[\\u4e00-\\u9fa5]{0,}$"),
    //中英文数字下划线
    CHAR_CHN_ENG_NUM(10, "^[\\u4E00-\\u9FA5A-Za-z0-9_]+$"),

    //复杂密码Should have 1 lowercase letter, 1 uppercase letter, 1 number, 1 special character and be at least 8 characters long
    COMPLEXPWD(11, "(?=(.*[0-9]))(?=.*[\\!@#$%^&*()\\\\[\\]{}\\-_+=~`|:;\"'<>,./?])(?=.*[a-z])(?=(.*[A-Z]))(?=(.*)).{8,}"),
    //中等强度密码Should have 1 lowercase letter, 1 uppercase letter, 1 number, and be at least 8 characters long
    MODERATEPWD(12, "(?=(.*[0-9]))((?=.*[A-Za-z0-9])(?=.*[A-Z])(?=.*[a-z]))^.{8,}$"),

    DATE_ISO8601_YYYY_MM_DD(13, "^(?![+-]?\\d{4,5}-?(?:\\d{2}|W\\d{2})T)(?:|(\\d{4}|[+-]\\d{5})-?(?:|(0\\d|1[0-2])(?:|-?([0-2]\\d|3[0-1]))|([0-2]\\d{2}|3[0-5]\\d|36[0-6])|W([0-4]\\d|5[0-3])(?:|-?([1-7])))(?:(?!\\d)|T(?=\\d)))(?:|([01]\\d|2[0-4])(?:|:?([0-5]\\d)(?:|:?([0-5]\\d)(?:|\\.(\\d{3})))(?:|[zZ]|([+-](?:[01]\\d|2[0-4]))(?:|:?([0-5]\\d)))))$"),
    DATE_FORMAT_DD_MM_YYYY(14, "^(0?[1-9]|[12][0-9]|3[01])([ \\/\\-])(0?[1-9]|1[012])\\2([0-9][0-9][0-9][0-9])(([ -])([0-1]?[0-9]|2[0-3]):[0-5]?[0-9]:[0-5]?[0-9])?$"),

    INTEGER(20, "^-?\\d+$"),
    NEGATIVE_INTEGER(21, "^-\\d*\\.?\\d+$"),
    POSITIVE_INTEGER(22, "^\\d+$"),
    ZERO_OR_POSITIVE_INTEGER(23, "^[1-9]\\d*|0$"),

    FLOAT(24, "^-?([1-9]\\d*\\.\\d*|0\\.\\d*[1-9]\\d*|0?\\.0+|0)$"),
    POSITIVE_FLOAT(25, "^[1-9]\\d*\\.\\d*|0\\.\\d*[1-9]\\d*$"),
    NEGATIVE_FLOAT(26, "^-([1-9]\\d*\\.\\d*|0\\.\\d*[1-9]\\d*)$"),
    ZERO_OR_POSITIVE_FLOAT(27, "^[1-9]\\d*\\.\\d*|0\\.\\d*[1-9]\\d*|0?\\.0+|0$"),
    ;

    private int id;
    private String expression;

    WellKnownRegex(int _id, String _expression) {
        this.id = _id;
        this.expression = _expression;
    }

    public int getId() {
        return id;
    }

    public String getExpression() {
        return expression;
    }
}

