package pps.core.system.module;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.common.utils.ManifestComponentInfoHelper;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.provider.impl.PpsPermissionProviderImpl;
import pps.core.system.provider.impl.PpsSsoProviderImpl;
import pps.core.system.provider.impl.PpsUserSessionInjectProviderImpl;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XModule;
import xstartup.base.pattern.XProviderManager;
import xstartup.base.util.XCopyUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.core.base.provider.spec.XPermissionProvider;
import xstartup.core.base.provider.spec.XSessionInjectProvider;
import xstartup.core.base.provider.spec.XSsoProvider;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@XText("系统模块")
public class PpsCoreSystemModule extends XModule {
    protected PpsCoreSystemModule() {
        super(1002);
    }

    @Override
    protected void onStarted(XContext context) {
        XProviderManager.setDefaultImplement(context, XPermissionProvider.class, PpsPermissionProviderImpl.class);
        XProviderManager.setDefaultImplement(context, XSsoProvider.class, PpsSsoProviderImpl.class);
        XProviderManager.setDefaultImplement(context, XSessionInjectProvider.class, PpsUserSessionInjectProviderImpl.class);
        //配置信息 自定义实现类
        ManifestComponentInfoHelper.init(this.getClass().getClassLoader());
        SysDictionaryMapper dictionaryMapper = context.getBean(SysDictionaryMapper.class);
        //初始化字典缓存
        //查询所有alias不为空的
        Map<String, List<SysDictionaryCache>> dictCashMap = new HashMap<>();
        List<SysDictionaryEnt> sysDictionarys = dictionaryMapper.selectList(new LambdaQueryWrapper<SysDictionaryEnt>()
                .isNotNull(SysDictionaryEnt::getAlias).gt(SysDictionaryEnt::getEndTime, DateUtil.date())
        );
        sysDictionarys = sysDictionarys.stream().filter(p -> !StringUtils.equals("", p.getAlias())).collect(Collectors.toList());
        for (SysDictionaryEnt dictObj : sysDictionarys) {
            List<SysDictionaryCache> list = new ArrayList<>();
            SysDictionaryCache dictMap = new SysDictionaryCache();
            XCopyUtils.copyObject(dictObj, dictMap);
            list.add(dictMap);
            dictCashMap.put(dictObj.getAlias(), list);
        }
        //所有alias不为空的字典
        List<String> dictIds = sysDictionarys.stream().map(SysDictionaryEnt::getId).distinct().collect(Collectors.toList());
        int size = dictIds.size();
        while (size > 0) {
            //根据pid查询所有子字典项
            List<SysDictionaryEnt> sysDictionaryChilds = dictionaryMapper.selectList(new LambdaQueryWrapper<SysDictionaryEnt>()
                    .in(SysDictionaryEnt::getPid, dictIds).gt(SysDictionaryEnt::getEndTime, DateUtil.date())
            );
            Set<String> aliasObjs = dictCashMap.keySet();
            //放入到缓存map中
            for (String aliasObj : aliasObjs) {
                List<SysDictionaryCache> sysDictionaryCaches = dictCashMap.get(aliasObj);
                Iterator<SysDictionaryCache> iterator = sysDictionaryCaches.iterator();
                if (iterator.hasNext()) {
                    if (ObjectUtils.isNotEmpty(sysDictionaryChilds)) {
                        SysDictionaryCache dict = iterator.next();
                        for (SysDictionaryEnt dictChild : sysDictionaryChilds) {
                            if (StringUtils.equals(dict.getId(), dictChild.getPid())) {
                                List<SysDictionaryCache> sysDictionaryList = dictCashMap.get(aliasObj);
                                SysDictionaryCache dictMap = new SysDictionaryCache();
                                XCopyUtils.copyObject(dictChild, dictMap);
                                sysDictionaryList.add(dictMap);
                                dictCashMap.put(aliasObj, sysDictionaryList);
                            }
                        }
                    }
                }
            }
            dictIds = sysDictionaryChilds.stream().map(SysDictionaryEnt::getId).distinct().collect(Collectors.toList());
            size = dictIds.size();
        }
        //放入到缓存redis
        Set<String> keys = dictCashMap.keySet();
        for (String key : keys) {
            List<SysDictionaryCache> sysDictionaryCaches = dictCashMap.get(key);
            SysDictionaryCache.deleteCache(context, key);
            SysDictionaryCache.set(context, key, sysDictionaryCaches);
        }
        XThreadHelper.async(() -> this.fixDicPath(context));

    }

    private void fixDicPath(XContext context) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(new QueryWrapper<>());
        final Map<String, SysDictionaryEnt> dics = sysDictionaryEnts.stream().collect(Collectors.toMap(SysDictionaryEnt::getId, Function.identity()));
        StringBuilder sb = new StringBuilder();
        sysDictionaryEnts.forEach(item -> {
            sb.setLength(0);
            if (CharSequenceUtil.isBlank(item.getPid())) {
                return;
            }
            if (CharSequenceUtil.isNotBlank(item.getDicPath())) {
                return;
            }

            buildPath(dics, item.getPid(), sb);
            String path = sb.toString();
            if (CharSequenceUtil.isBlank(path)) {
                return;
            }

            if (path.equals(item.getDicPath())) {
                return;
            }

            SysDictionaryEnt dicForUpdate = new SysDictionaryEnt();
            dicForUpdate.setId(item.getId());
            dicForUpdate.setDicPath(path);
            mapper.updateById(dicForUpdate);
        });

    }

    private void buildPath(Map<String, SysDictionaryEnt> dics, String parentId, StringBuilder sb) {
        if (CharSequenceUtil.isBlank(parentId) || "1".equals(parentId)) {
            return;
        }

        sb.insert(0, parentId);
        sb.insert(0, "/");
        if (!dics.containsKey(parentId)) {
            return;
        }

        SysDictionaryEnt parent = dics.get(parentId);
        if (parent == null) {
            return;
        }

        buildPath(dics, parent.getPid(), sb);

    }
}
