package pps.core.system.provider.impl;

import pps.core.system.cache.RolePermissionCache;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.XUser;
import xstartup.base.dict.XMatchModeDict;
import xstartup.base.metadata.XActionDefine;
import xstartup.core.base.provider.spec.XPermissionProvider;
import xstartup.data.XServiceResult;
import xstartup.error.XError;

@XImplement
public class PpsPermissionProviderImpl implements XPermissionProvider {
    @Override
    public XServiceResult checkPermission(XContext context, XActionDefine actionDefine, XUser user) {
        if (actionDefine.getPermission() == null) {
            context.getLogger().debug("permission none,result ok");
            return XServiceResult.OK;
        }

        return checkPermission(context, actionDefine.getPermission().getCodes(),
                actionDefine.getPermission().getMatchMode(),
                user);
    }

    @Override
    public XServiceResult checkPermission(XContext context, String[] codes, XMatchModeDict matchMode, XUser user) {
        Long[] roleIds = user.getRoles();
        if (matchMode.equals(XMatchModeDict.Any)) {
            for (String permission : codes) {
                for (Long roleId : roleIds) {
                    if (RolePermissionCache.exist(context, roleId, permission)) {
                        context.getLogger().debug("permission any,exist:{}", permission);
                        context.getLogger().debug("permission any,result ok");
                        return XServiceResult.OK;
                    } else {
                        context.getLogger().debug("permission any,not exist:{}", permission);
                    }
                }
            }
            context.getLogger().debug("permission any,result deny");
            return XServiceResult.error(context, XError.Deny);
        } else {
            for (String permission : codes) {
                for (Long roleId : roleIds) {
                    if (RolePermissionCache.exist(context, roleId, permission)) {
                        context.getLogger().debug("permission all,exist:{}", permission);
                    } else {
                        context.getLogger().debug("permission all,not exist:{}", permission);
                        context.getLogger().debug("permission all,result deny");
                        return XServiceResult.error(context, XError.Deny);
                    }
                }
            }
            context.getLogger().info("permission all,result ok");
            return XServiceResult.OK;
        }
    }
}
