package pps.core.system.service;

import cn.hutool.extra.pinyin.PinyinUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.system.cache.SysDictionaryCache;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysDictionaryView;
import pps.core.system.entity.SysOrganizationDictionaryEnt;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import pps.core.system.mapper.SysOrganizationDictionaryMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import pps.core.system.service.data.sys_dictionary.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.data.XPageInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@XText("字典接口")
@XService
public class SysDictionaryService {
    @XText("创建字典")
    @XApiPost
    public XServiceResult createSysDictionary(XContext context, CreateSysDictionaryInput input) {
        return XTransactionHelper.begin(context, () -> {
            SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
            //如果输入的 alias 为空，查询新增项的上级alias
            String alias;
            if (StringUtils.isNotBlank(input.getAlias())) {
                alias = input.getAlias();
                List<SysDictionaryCache> list1 = SysDictionaryCache.list(context, alias);
                if (ObjectUtils.isNotEmpty(list1)) {
                    return XServiceResult.error(200, "字典标识已存在！");
                }
            } else {
                alias = selectSysDictionaryAlias(context, input.getPid());
            }
            boolean exist = SysDictionaryCache.exist(context, alias, input.getDicKey());
            if (exist) {
                return XServiceResult.error(200, "字典编码已存在！");
            } else {
                //缓存不存在数据库查询
                QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysDictionaryEnt::getDicKey, input.getDicKey())
                        .gt(SysDictionaryEnt::getEndTime, new Date());
                if (StringUtils.isNotBlank(input.getPid())) {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                } else {
                    queryWrapper.lambda().eq(SysDictionaryEnt::getPid, "1");
                }
                SysDictionaryEnt ent = mapper.selectOne(queryWrapper);
                if (ent != null) {
                    return XServiceResult.error(200, "字典编码已存在！");
                }
            }
            SysDictionaryEnt entity = new SysDictionaryEnt();
            String dicId = UUID.randomUUID().toString();
            XCopyUtils.copyObject(input, entity);
            if (StringUtils.equals("1", input.getPid())) {
                if (StringUtils.isBlank(input.getAlias())) {
                    return XServiceResult.error(200, "新增一级字典，字典标识不能为空！");
                }
                entity.setPid("1");
                entity.setDicPath(null);
            } else {
                QueryWrapper<SysDictionaryEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysDictionaryEnt::getId, input.getPid());
                SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(wrapper);
                //查看有没有dicPath,如果没有说明是一级，直接dicPath=/父级id
                if (ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                    if (StringUtils.isNotBlank(sysDictionaryEnt.getDicPath())) {
                        entity.setDicPath(sysDictionaryEnt.getDicPath() + "/" + sysDictionaryEnt.getId());
                    } else {
                        entity.setDicPath("/" + sysDictionaryEnt.getId());
                    }
                }
            }
            if (input.getSort() == null) {
                QueryWrapper<SysDictionaryEnt> queryMaxSortEntity = new QueryWrapper<>();
                queryMaxSortEntity.lambda().gt(SysDictionaryEnt::getEndTime, new Date());
                if (StringUtils.isBlank(input.getPid())) {
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, "1")
                            .isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                } else {
                    queryMaxSortEntity.lambda().eq(SysDictionaryEnt::getPid, input.getPid());
                }
                //查询最大的sort值
                queryMaxSortEntity.lambda().isNotNull(SysDictionaryEnt::getSort).orderByDesc(SysDictionaryEnt::getSort);
                List<SysDictionaryEnt> sysDictionarys = mapper.selectList(queryMaxSortEntity);
                if (ObjectUtils.isNotEmpty(sysDictionarys)) {
                    SysDictionaryEnt sysDictionaryEnt1 = sysDictionarys.get(0);
                    if (sysDictionaryEnt1.getSort() != null) {
                        entity.setSort(sysDictionaryEnt1.getSort() + 1);
                    } else {
                        entity.setSort(1);
                    }
                } else {
                    entity.setSort(1);
                }
            }
            entity.setEffectTime(new Date());
            entity.setId(dicId);
//        entity.setcr(context.getSession(PpsUserSession.class).getId());
            entity.setCreateName(context.getUser().getName());
            entity.setCreateTime(new Date());
            mapper.insert(entity);
            //更新字典缓存
            if (entity.getEndTime() != null) {
                long l = entity.getEndTime().getTime() - System.currentTimeMillis();
                if (l > 0) {
                    SysDictionaryCache.deleteCache(context, alias);
                    QueryWrapper<SysDictionaryEnt> queryAlias = new QueryWrapper<>();
                    queryAlias.lambda().eq(SysDictionaryEnt::getAlias, alias).gt(SysDictionaryEnt::getEndTime, new Date());
                    SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryAlias);
                    if (ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                        QueryWrapper<SysDictionaryEnt> queryAll = new QueryWrapper<>();
                        queryAll.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryEnt.getId()).gt(SysDictionaryEnt::getEndTime, new Date());
                        List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(queryAll);
                        if (ObjectUtils.isNotEmpty(sysDictionaryEnts)) {
                            sysDictionaryEnts.add(entity);
                        }
                        sysDictionaryEnts.add(sysDictionaryEnt);
                        List<SysDictionaryCache> sysDictionaryCaches = XCopyUtils.copyNewList(sysDictionaryEnts, SysDictionaryCache.class);
                        SysDictionaryCache.set(context, alias, sysDictionaryCaches);
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    //递归查询当前项最近的alias
    public String selectSysDictionaryAlias(XContext context, String pid) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        String alias = "";
        while (StringUtils.isBlank(alias)) {
            QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysDictionaryEnt::getId, pid);
            SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
            if (ObjectUtils.isNotEmpty(sysDictionaryEnt)) {
                if (StringUtils.isNotBlank(sysDictionaryEnt.getAlias())) {
                    alias = sysDictionaryEnt.getAlias();
                } else {
                    pid = sysDictionaryEnt.getPid();
                }
            } else {
                return "";
            }
        }
        return alias;
    }

    @XText("更新字典")
    @XApiPost
    public XServiceResult updateSysDictionary(XContext context, UpdateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        //如果输入的alias为空，查询新增项的上级alias
        String alias;
        if (StringUtils.isNotBlank(input.getAlias())) {
            alias = input.getAlias();
            List<SysDictionaryCache> list1 = SysDictionaryCache.list(context, alias);
            if (ObjectUtils.isNotEmpty(list1)) {
                boolean flag = true;
                for (SysDictionaryCache cash : list1) {
                    if (StringUtils.equals(cash.getId(), input.getId()) && StringUtils.equals(alias, cash.getAlias())) {
                        flag = false;
                    }
                }
                if (flag) {
                    return XServiceResult.error(200, "字典编码已存在！");
                }
            }
        } else {
            alias = selectSysDictionaryAlias(context, input.getPid());
            boolean exist = SysDictionaryCache.exist(context, alias, input.getDicKey());
            if (exist) {
                List<SysDictionaryCache> list1 = SysDictionaryCache.list(context, alias);
                if (ObjectUtils.isNotEmpty(list1)) {
                    boolean flag = true;
                    for (SysDictionaryCache cash : list1) {
                        if (StringUtils.equals(cash.getId(), input.getId())) {
                            flag = false;
                        }
                    }
                    if (flag) {
                        return XServiceResult.error(200, "字典编码已存在！");
                    }
                }
            }
        }
        //查询需要修改的
        QueryWrapper<SysDictionaryEnt> queryEntity = new QueryWrapper<>();
        queryEntity.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt entity = mapper.selectOne(queryEntity);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        entity.setModifyName(context.getUser().getName());
        entity.setModifyTime(new Date());
        mapper.updateById(entity);
        //更新字典缓存
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, alias);
        List<SysDictionaryCache> items = new ArrayList<>();
        for (SysDictionaryCache dict : list) {
            if (StringUtils.equals(entity.getId(), dict.getId())) {
                SysDictionaryCache.delete(context, alias, entity.getId());
                //更新字典缓存
                if (entity.getEndTime() != null) {
                    long l = entity.getEndTime().getTime() - new Date().getTime();
                    if (l > 0) {
                        SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                        XCopyUtils.copyObject(entity, sysDictionaryCache);
                        items.add(sysDictionaryCache);
                        continue;
                    }
                }
            }
            items.add(dict);
        }
        SysDictionaryCache.set(context, alias, items);
        return XServiceResult.OK;
    }

    @XText("删除字典")
    @XApiPost
    public XServiceResult deleteSysDictionary(XContext context, DeleteSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        SysDictionaryEnt sysDictionaryEnt = new SysDictionaryEnt();
        sysDictionaryEnt.setId(entity.getId());
        sysDictionaryEnt.setEndTime(new Date());
        mapper.updateById(sysDictionaryEnt);
        if (StringUtils.isBlank(entity.getAlias())) {
            String alias = selectSysDictionaryAlias(context, entity.getPid());
            SysDictionaryCache.delete(context, alias, entity.getId());
        } else {
            SysDictionaryCache.delete(context, entity.getAlias(), entity.getId());
        }
        return XServiceResult.OK;
    }

    @XText("根据id查询字典")
    @XApiPost
    public XSingleResult<GetSysDictionaryOutput> getSysDictionary(XContext context, GetSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysDictionaryOutput output = new GetSysDictionaryOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页条件查询字典")
    @XApiPost
    public XPageResult<QuerySysDictionaryOutput> querySysDictionary(XContext context, QuerySysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysDictionaryViewMapper viewMapper = context.getBean(SysDictionaryViewMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().gt(SysDictionaryEnt::getEndTime, new Date());
        if (StringUtils.isNotBlank(input.getId())) {
            queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        }
        if (StringUtils.isNotBlank(input.getParentId())) {
            queryWrapper.lambda().eq(SysDictionaryEnt::getPid, input.getParentId());
            if (StringUtils.equals("1", input.getParentId())) {
                QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
                if (StringUtils.isNotBlank(input.getCode())) {
                    query.lambda().like(SysDictionaryEnt::getDicKey, input.getCode());
                }
                if (StringUtils.isNotBlank(input.getDicName())) {
                    query.lambda().like(SysDictionaryEnt::getDicName, input.getDicName());
                }
                List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(query);
                if (ObjectUtils.isNotEmpty(sysDictionaryEnts)) {
                    List<String> collect = sysDictionaryEnts.stream().map(p -> {
                        if (StringUtils.isNotBlank(p.getDicPath())) {
                            String[] split = p.getDicPath().split("/");
                            if (ObjectUtils.isNotEmpty(split) && split.length > 1) {
                                return split[1];
                            } else {
                                return null;
                            }
                        } else {
                            return p.getId();
                        }
                    }).distinct().collect(Collectors.toList());
                    queryWrapper.lambda().in(SysDictionaryEnt::getId, collect);
                } else {
                    return XPageResult.success(new ArrayList<>(), 1, 1, 0);
                }
            } else {
                input.setPage(1);
                input.setLimit(2000);
            }
        }
        if (input.getDicType() != null) {
            queryWrapper.lambda().eq(SysDictionaryEnt::getDicType, input.getDicType());
        }
        queryWrapper.lambda().orderByAsc(SysDictionaryEnt::getSort);
        XPageResult<QuerySysDictionaryOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QuerySysDictionaryOutput.class);
        List<QuerySysDictionaryOutput> items = query.getResult().getItems();
        if (items != null && items.size() > 0) {
            List<String> ids = items.stream().map(p -> {
                return p.getId();
            }).distinct().collect(Collectors.toList());
            SysDictionaryView sysDictionaryView = new SysDictionaryView();
            StringBuffer idsStr = new StringBuffer();
            for (int i = 0; i < ids.size(); i++) {
                if (i == 0) {
                    idsStr.append("'" + ids.get(i) + "'");
                } else {
                    idsStr.append(",'" + ids.get(i) + "'");
                }
            }
            sysDictionaryView.setPid(idsStr.toString());
            sysDictionaryView.setEndTime(new Date());
            List<SysDictionaryView> sysDictionaryViews = viewMapper.selectListsByParentIds(sysDictionaryView);
            int i = 0;
            for (QuerySysDictionaryOutput out : items) {
                if (sysDictionaryViews != null && sysDictionaryViews.size() > 0) {
                    boolean flag = false;
                    for (SysDictionaryView view : sysDictionaryViews) {
                        if (StringUtils.equals(out.getId(), view.getPid())) {
                            flag = true;
                            break;
                        }
                    }
                    if (!flag) {
                        //添加子节点是否打开的标志
                        out.setChildren(null);
                    }
                } else {
                    //添加子节点是否打开的标志
                    out.setChildren(null);
                }
                out.setXh(++i + "");
            }
        }
        return query;
    }

    @XText("分页条件查询字典")
    @XApiPost
    public XPageResult<QuerySysDictionaryOutput> querySysDictionaryOn(XContext context, QuerySysDictionaryInput input) {
        if (XStringUtils.isNotEmpty(input.getCode()) || XStringUtils.isNotEmpty(input.getDicName())) {
            SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
            SysDictionaryView view = new SysDictionaryView();
            if (StringUtils.isNotBlank(input.getCode())) {
                view.setDicKey(input.getCode());
            }
            if (StringUtils.isNotBlank(input.getDicName())) {
                view.setDicName(input.getDicName());
            }
            List<SysDictionaryView> viewList = mapper.selectTreeList(view);
            List<QuerySysDictionaryOutput> list = XCopyUtils.copyNewList(viewList, QuerySysDictionaryOutput.class);
            List<QuerySysDictionaryOutput> trees = new ArrayList<>();
            int i = 0;
            for (QuerySysDictionaryOutput tree : list) {
                tree.setBeHasChildren(null);
                tree.setBeDefault(null);
                tree.set_showChildren(null);
                if (tree.getSort() == null)
                    tree.setSort(0);
                tree.setChildren(null);
                if (tree.getPid() == null || "".equals(tree.getPid()) || "1".equals(tree.getPid())) {
                    trees.add(tree);
                    tree.setXh(++i + "");
                }
                //子集
                int j = 0;
                for (QuerySysDictionaryOutput it : list) {
                    if (tree.getId().equals(it.getPid())) {
                        if (tree.getChildren() == null) {
                            tree.setChildren(new ArrayList<>());
                        }
                        it.setXh(++j + "");
                        tree.set_showChildren(true);
                        tree.setBeHasChildren(1);
                        tree.getChildren().add(it);
                        tree.setBeDefault(1);
                    }
                }
                if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                    List<QuerySysDictionaryOutput> children = tree.getChildren();
                    children = children.stream().sorted(Comparator.comparing(QuerySysDictionaryOutput::getXh)).collect(Collectors.toList());
                    tree.setChildren(children);
                }
            }
            trees = trees.stream().sorted(Comparator.comparing(QuerySysDictionaryOutput::getSort)).collect(Collectors.toList());
            PageInfo<QuerySysDictionaryOutput> pageInfo = new PageInfo<>(trees);
            return XPageResult.success(trees, input, pageInfo.getTotal());
        } else {
            SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
            SysDictionaryViewMapper viewMapper = context.getBean(SysDictionaryViewMapper.class);
            QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().gt(SysDictionaryEnt::getEndTime, new Date());
            if (StringUtils.isNotBlank(input.getId())) {
                queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
            }
            if (StringUtils.isNotBlank(input.getParentId())) {
                queryWrapper.lambda().eq(SysDictionaryEnt::getPid, input.getParentId());
                if (StringUtils.equals("1", input.getParentId())) {
                    QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
                    if (StringUtils.isNotBlank(input.getCode())) {
                        query.lambda().like(SysDictionaryEnt::getDicKey, input.getCode());
                    }
                    if (StringUtils.isNotBlank(input.getDicName())) {
                        query.lambda().like(SysDictionaryEnt::getDicName, input.getDicName());
                    }
                    List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(query);
                    if (ObjectUtils.isNotEmpty(sysDictionaryEnts)) {
                        List<String> collect = sysDictionaryEnts.stream().map(p -> {
                            if (StringUtils.isNotBlank(p.getDicPath())) {
                                String[] split = p.getDicPath().split("/");
                                if (ObjectUtils.isNotEmpty(split) && split.length > 1) {
                                    return split[1];
                                } else {
                                    return null;
                                }
                            } else {
                                return p.getId();
                            }
                        }).distinct().collect(Collectors.toList());
                        queryWrapper.lambda().in(SysDictionaryEnt::getId, collect);
                    } else {
                        return XPageResult.success(new ArrayList<>(), 1, 1, 0);
                    }
                } else {
                    input.setPage(1);
                    input.setLimit(2000);
                }
            }
            if (input.getDicType() != null) {
                queryWrapper.lambda().eq(SysDictionaryEnt::getDicType, input.getDicType());
            }
            queryWrapper.lambda().orderByAsc(SysDictionaryEnt::getSort);
            XPageResult<QuerySysDictionaryOutput> query = XMapperHelper.query(mapper, input, queryWrapper, QuerySysDictionaryOutput.class);
            List<QuerySysDictionaryOutput> items = query.getResult().getItems();
            if (items != null && items.size() > 0) {
                List<String> ids = items.stream().map(p ->
                        p.getId()
                ).distinct().collect(Collectors.toList());
                SysDictionaryView sysDictionaryView = new SysDictionaryView();
                StringBuffer idsStr = new StringBuffer();
                for (int i = 0; i < ids.size(); i++) {
                    if (i == 0) {
                        idsStr.append("'" + ids.get(i) + "'");
                    } else {
                        idsStr.append(",'" + ids.get(i) + "'");
                    }
                }
                sysDictionaryView.setPid(idsStr.toString());
                sysDictionaryView.setEndTime(new Date());
                List<SysDictionaryView> sysDictionaryViews = viewMapper.selectListsByParentIds(sysDictionaryView);
                int i = 0;
                for (QuerySysDictionaryOutput out : items) {
                    if (sysDictionaryViews != null && sysDictionaryViews.size() > 0) {
                        boolean flag = false;
                        for (SysDictionaryView view : sysDictionaryViews) {
                            if (StringUtils.equals(out.getId(), view.getPid())) {
                                flag = true;
                                break;
                            }
                        }
                        if (!flag) {
                            //添加子节点是否打开的标志
                            out.setChildren(null);
                        }
                    } else {
                        //添加子节点是否打开的标志
                        out.setChildren(null);
                    }
                    out.setXh(++i + "");
                }
            }
            return query;
        }
    }

    @XText("查询字典列表树")
    @XApiPost
    public XListResult<QuerySysDictionaryOutput> getSysDictionaryTree(XContext context, QuerySysDictionaryInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        if (XStringUtils.isEmpty(input.getCode()) && XStringUtils.isEmpty(input.getDicName()))
            return XListResult.error(500, "请输入查询参数");
        SysDictionaryView view = new SysDictionaryView();
        if (StringUtils.isNotBlank(input.getCode())) {
            view.setDicKey(input.getCode());
        }
        if (StringUtils.isNotBlank(input.getDicName())) {
            view.setDicName(input.getDicName());
        }
        List<SysDictionaryView> viewList = mapper.selectTreeList(view);
        List<QuerySysDictionaryOutput> list = XCopyUtils.copyNewList(viewList, QuerySysDictionaryOutput.class);
        List<QuerySysDictionaryOutput> trees = new ArrayList<>();
        for (QuerySysDictionaryOutput tree : list) {
            if (tree.getPid() == null || "".equals(tree.getPid()) || "1".equals(tree.getPid())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysDictionaryOutput it : list) {
                if (tree.getId().equals(it.getPid())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (tree.getChildren() != null && tree.getChildren().size() > 1) {
                List<QuerySysDictionaryOutput> children = tree.getChildren();
                children = children.stream().sorted(Comparator.comparing(QuerySysDictionaryOutput::getSort)).collect(Collectors.toList());
                tree.setChildren(children);
            }
        }
        trees = trees.stream().sorted(Comparator.comparing(QuerySysDictionaryOutput::getSort)).collect(Collectors.toList());
        return XListResult.success(trees);
    }

    @XApiPost
    public XPageResult<QuerySysDictionaryOutput> dynamicQuerySysDictionary(XContext context, CustomQueryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        return XMapperHelper.query(mapper, input, SysDictionaryEnt.class, QuerySysDictionaryOutput.class);
    }

    @XApiPost
    public XSingleResult<GetSysDictionaryViewOutput> getSysDictionaryView(XContext context, GetSysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        SysDictionaryView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysDictionaryViewOutput output = new GetSysDictionaryViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QuerySysDictionaryViewOutput> querySysDictionaryView(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysDictionaryView> list = mapper.selectList(record);
        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>(list);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据父级id(可以多个id)获取字典")
    @XApiPost
    public XPageResult<QuerySysDictionaryViewOutput> querySysDictionarysByParentIds(XContext context, QuerySysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysDictionaryView> list = mapper.selectListsByParentIds(record);
        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>(list);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据alias获取字典项列表")
    @XApiPost
    public XListResult<QuerySysDictionaryViewOutput> querySysDictionarysByAlias(XContext context, CreateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysOrganizationDictionaryMapper orgDicmapper = context.getBean(SysOrganizationDictionaryMapper.class);
        SysOrganizationRelMapper orgRelMapper = context.getBean(SysOrganizationRelMapper.class);
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, input.getAlias());
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias, input.getAlias()).gt(SysDictionaryEnt::getEndTime, new Date());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        list = list.stream().filter((SysDictionaryCache dic) -> StringUtils.equals(dic.getPid(), sysDictionaryEnt.getId()))
                .collect(Collectors.toList());
        if (ObjectUtils.isEmpty(list)) {
            list = SysDictionaryCache.list(context, input.getAlias());
            //查询alias字典
            QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
            query.lambda().eq(SysDictionaryEnt::getAlias, input.getAlias()).gt(SysDictionaryEnt::getEndTime, new Date());
            SysDictionaryEnt sysDictionary = mapper.selectOne(query);
            if (ObjectUtils.isNotEmpty(sysDictionary)) {
                list = new ArrayList<>();
                SysDictionaryCache.set(context, input.getAlias(), list);
                SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                XCopyUtils.copyObject(sysDictionary, sysDictionaryCache);
                list.add(sysDictionaryCache);
                //查询alias字典下级字典项
                QueryWrapper<SysDictionaryEnt> queryItems = new QueryWrapper();
                //如果字典项在单位属性字典配置了，需要查询单位属性字典
                if (ObjectUtils.isNotEmpty(sysDictionary.getIsOrg()) && sysDictionary.getIsOrg() == 1) {
                    //查询当前登录人的组织机构层级路径
                    QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
                    queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                            .gt(SysOrganizationRelEnt::getEndTime, new Date());
                    SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
                    //根据字典id查询
                    QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
                    queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionary.getId())
                            .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
                    List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
                    String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
                    String ouId = sysOrganizationRelEnt.getOuId();
                    if (StringUtils.isNotBlank(ouIdPath)) {
                        List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                            if (ouIdPath.indexOf(p.getOuId()) != -1 && p.getUseLowerLevel() == 1) {
                                return p.getDicCode();
                            } else if (ouId.indexOf(p.getOuId()) != -1) {
                                return p.getDicCode();
                            } else {
                                return "1";
                            }
                        }).collect(Collectors.toList());
                        queryItems.lambda().in(SysDictionaryEnt::getId, dicIds);
                    }
                }
                queryItems.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryCache.getId()).gt(SysDictionaryEnt::getEndTime, new Date());
                List<SysDictionaryEnt> sysDictionaryList = mapper.selectList(queryItems);
                if (ObjectUtils.isNotEmpty(sysDictionaryList)) {
                    if (ObjectUtils.isNotEmpty(list)) {
                        for (SysDictionaryEnt ent : sysDictionaryList) {
                            SysDictionaryCache cache = new SysDictionaryCache();
                            XCopyUtils.copyObject(ent, cache);
                            list.add(cache);
                        }
                    }
                    Collections.sort(sysDictionaryList, (o1, o2) -> {
                        if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                            return o1.getSort() - o2.getSort();
                        } else {
                            return 0;
                        }
                    });
                    List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(sysDictionaryList, QuerySysDictionaryViewOutput.class);
                    return XListResult.success(outputs);
                }
            }
        }
        if (ObjectUtils.isNotEmpty(sysDictionaryEnt.getIsOrg()) && sysDictionaryEnt.getIsOrg() == 1) {
            //查询当前登录人的组织机构层级路径
            QueryWrapper<SysOrganizationRelEnt> queryOrgRel = new QueryWrapper<>();
            queryOrgRel.lambda().eq(SysOrganizationRelEnt::getOuId, input.getOuId())
                    .gt(SysOrganizationRelEnt::getEndTime, new Date());
            SysOrganizationRelEnt sysOrganizationRelEnt = orgRelMapper.selectOne(queryOrgRel);
            //根据字典id查询
            QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
            queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionaryEnt.getId())
                    .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
            List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
            String ouIdPath = sysOrganizationRelEnt.getOuIdPath();
            String ouId = sysOrganizationRelEnt.getOuId();
            if (StringUtils.isNotBlank(ouIdPath)) {
                List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p -> {
                    if (ouIdPath.indexOf(p.getOuId()) != -1 && p.getUseLowerLevel() == 1) {
                        return p.getDicCode();
                    } else if (ouId.indexOf(p.getOuId()) != -1) {
                        return p.getDicCode();
                    } else {
                        return "1";
                    }
                }).collect(Collectors.toList());
                list = list.stream().filter(i -> dicIds.contains(i.getId())).collect(Collectors.toList());
            }
        }
        Collections.sort(list, (o1, o2) -> {
            if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                return o1.getSort() - o2.getSort();
            } else {
                return 0;
            }
        });
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("获取字典地区树")
    @XApiPost
    public XListResult<QuerySysDictionaryViewOutput> querySysDictionaryTree(XContext context, GetSysDictionaryViewInput input) {
        SysDictionaryViewMapper mapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView record = new SysDictionaryView();
        XCopyUtils.copyObject(input, record);
        List<SysDictionaryView> list = mapper.selectSysDictionaryTree(record);
        List<QuerySysDictionaryViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        List<QuerySysDictionaryViewOutput> trees = new ArrayList<>();
        for (QuerySysDictionaryViewOutput tree : outputs) {
            if (tree.getParentSysDictionaryId().equals("30")) {
                trees.add(tree);
            }
            //子集
            for (QuerySysDictionaryViewOutput it : outputs) {
                if (tree.getId().equals(it.getParentSysDictionaryId())) {
                    if (tree.getChildren() == null) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
        }
        trees.stream().sorted(Comparator.comparing(QuerySysDictionaryViewOutput::getSort)).collect(Collectors.toList());
        return XListResult.success(trees);
    }

    @XText("初始化字典dicPath")
    @XApiPost
    public XServiceResult updateBasePipelinePath(XContext context, UpdateSysDictionaryInput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().and(wrapper -> wrapper.eq(SysDictionaryEnt::getPid, "1").or().eq(SysDictionaryEnt::getPid, null)
                .or().eq(SysDictionaryEnt::getPid, "2").or().eq(SysDictionaryEnt::getPid, "20"));
        XPageInput page = new XPageInput();
        page.setPage(1);
        page.setLimit(100000);
        IPage<SysDictionaryEnt> opsPipelineEnts = XMapperHelper.queryPage(mapper, page, queryWrapper);
        List<SysDictionaryEnt> records = opsPipelineEnts.getRecords();
        updateBasePipelineParentPath(context, records);
        return XServiceResult.OK;
    }

    public XServiceResult updateBasePipelineParentPath(XContext context, List<SysDictionaryEnt> dics) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        List<String> list = dics.stream().map(p ->
                p.getId()
        ).distinct().collect(Collectors.toList());
        while (ObjectUtils.isNotEmpty(list)) {
            //更新二级字典parentPath
            QueryWrapper<SysDictionaryEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().in(SysDictionaryEnt::getPid, list);
            List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(wrapper);
            if (ObjectUtils.isNotEmpty(sysDictionaryEnts)) {
                for (SysDictionaryEnt entity : sysDictionaryEnts) {
                    QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().in(SysDictionaryEnt::getId, entity.getPid());
                    SysDictionaryEnt pdic = mapper.selectOne(queryWrapper);
                    String parentPath = pdic.getDicPath() == null ? "/" + pdic.getId() : pdic.getDicPath() + "/" + pdic.getId();
//                    String parentPath = entity.getDicPath() == null || entity.getDicPath() == "null" ? "/" + entity.getPid() : entity.getDicPath() + "/" + childDic.getId();
                    entity.setDicPath(parentPath);
                    mapper.updateById(entity);
                }
                list = sysDictionaryEnts.stream().map(p ->
                        p.getId()
                ).distinct().collect(Collectors.toList());
            } else {
                list = null;
            }
        }
        return XServiceResult.OK;
    }

    @XText("修改字典排序")
    @XApiPost
    public XServiceResult updateDictionarySortById(XContext context, QuerySysDictionaryOutput input) {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
        //查询当前字典项
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper();
        queryWrapper.lambda().eq(SysDictionaryEnt::getId, input.getId());
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        //查询上一个或者下一个字典项
        query.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryEnt.getPid());
        if (StringUtils.equals("up", input.getSortFlag())) {
            query.lambda().lt(SysDictionaryEnt::getSort, sysDictionaryEnt.getSort())
                    .orderByDesc(SysDictionaryEnt::getSort);
        } else {
            query.lambda().gt(SysDictionaryEnt::getSort, sysDictionaryEnt.getSort())
                    .orderByAsc(SysDictionaryEnt::getSort);
        }
        List<SysDictionaryEnt> sysDictionaryEnts = mapper.selectList(query);
        Integer sort;
        if (ObjectUtils.isNotEmpty(sysDictionaryEnts)) {
            SysDictionaryEnt sysDictionaryEnt1 = sysDictionaryEnts.get(0);
            //交换排序值
            sort = sysDictionaryEnt1.getSort();
            sysDictionaryEnt1.setSort(sysDictionaryEnt.getSort());
            mapper.updateById(sysDictionaryEnt1);
        } else {
            sort = sysDictionaryEnt.getSort();
        }
        sysDictionaryEnt.setSort(sort);
        mapper.updateById(sysDictionaryEnt);
        return XServiceResult.OK;
    }

    @XText("查询规则编码")
    @XApiPost
    public XPageResult<QueryRuleNumberOutput> queryRuleNumber(XContext context, QueryRuleNumberInput input) {
        final String RULE_NUMBER_ALIAS = "rule_number";
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        SysOrganizationDictionaryMapper orgDicmapper = context.getBean(SysOrganizationDictionaryMapper.class);
        List<SysDictionaryCache> list = SysDictionaryCache.list(context, RULE_NUMBER_ALIAS);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias, RULE_NUMBER_ALIAS);
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        list = list.stream().filter((SysDictionaryCache dic) -> StringUtils.equals(dic.getPid(), sysDictionaryEnt.getId()))
                .collect(Collectors.toList());
        if (ObjectUtils.isEmpty(list)) {
            list = SysDictionaryCache.list(context, RULE_NUMBER_ALIAS);
            //查询alias字典
            QueryWrapper<SysDictionaryEnt> query = new QueryWrapper();
            query.lambda().eq(SysDictionaryEnt::getAlias, RULE_NUMBER_ALIAS).gt(SysDictionaryEnt::getEndTime, new Date());
            SysDictionaryEnt sysDictionary = mapper.selectOne(query);
            if (ObjectUtils.isNotEmpty(sysDictionary)) {
                list = new ArrayList<>();
                SysDictionaryCache.set(context, RULE_NUMBER_ALIAS, list);
                SysDictionaryCache sysDictionaryCache = new SysDictionaryCache();
                XCopyUtils.copyObject(sysDictionary, sysDictionaryCache);
                list.add(sysDictionaryCache);
                //查询alias字典下级字典项
                QueryWrapper<SysDictionaryEnt> queryItems = new QueryWrapper();
                if (ObjectUtils.isNotEmpty(sysDictionary.getIsOrg()) && sysDictionary.getIsOrg() == 1) {
                    QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
                    queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getOuId, RULE_NUMBER_ALIAS)
                            .eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionary.getId()).gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
                    List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
                    List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p ->
                            p.getDicCode()
                    ).collect(Collectors.toList());
                    queryItems.lambda().in(SysDictionaryEnt::getId, dicIds);
                }
                queryItems.lambda().eq(SysDictionaryEnt::getPid, sysDictionaryCache.getId()).gt(SysDictionaryEnt::getEndTime, new Date());
                List<SysDictionaryEnt> sysDictionaryList = mapper.selectList(queryItems);
                if (ObjectUtils.isNotEmpty(sysDictionaryList)) {
                    if (ObjectUtils.isNotEmpty(list)) {
                        for (SysDictionaryEnt ent : sysDictionaryList) {
                            SysDictionaryCache cache = new SysDictionaryCache();
                            XCopyUtils.copyObject(ent, cache);
                            list.add(cache);
                        }
                    }
                    Collections.sort(list, (o1, o2) -> {
                        if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                            return o1.getSort() - o2.getSort();
                        } else {
                            return o1.getId().compareTo(o2.getId());
                        }
                    });
                    List<QueryRuleNumberOutput> outputs = XCopyUtils.copyNewList(sysDictionaryList, QueryRuleNumberOutput.class);
                    PageInfo<QueryRuleNumberOutput> pageInfo = new PageInfo<>(outputs);
                    return XPageResult.success(outputs, input, pageInfo.getTotal());
                }
            }
        }
        if (ObjectUtils.isNotEmpty(sysDictionaryEnt.getIsOrg()) && sysDictionaryEnt.getIsOrg() == 1) {
            QueryWrapper<SysOrganizationDictionaryEnt> queryOrgDic = new QueryWrapper();
            queryOrgDic.lambda().eq(SysOrganizationDictionaryEnt::getOuId, sysDictionaryEnt.getId())
                    .eq(SysOrganizationDictionaryEnt::getParentDicCode, sysDictionaryEnt.getId()).gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
            List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = orgDicmapper.selectList(queryOrgDic);
            List<String> dicIds = sysOrganizationDictionaryEnts.stream().map(p ->
                    p.getDicCode()
            ).collect(Collectors.toList());
            list = list.stream().filter(i -> dicIds.contains(i.getId())).collect(Collectors.toList());
        }
        Collections.sort(list, (o1, o2) -> {
            if (ObjectUtils.isNotEmpty(o1.getSort()) && ObjectUtils.isNotEmpty(o2.getSort())) {
                return o1.getSort() - o2.getSort();
            } else {
                return 0;
            }
        });
        List<QueryRuleNumberOutput> outputs = new ArrayList<>();
        //XCopyUtils.copyNewList(list, QuerySysDictionaryViewOutput.class);
        for (SysDictionaryCache sysDictionaryCache : list) {
            QueryRuleNumberOutput querySysDictionaryViewOutput = XCopyUtils.copyNewObject(sysDictionaryCache, QueryRuleNumberOutput.class);
            outputs.add(querySysDictionaryViewOutput);
        }
        PageInfo<QueryRuleNumberOutput> pageInfo = new PageInfo<>(outputs);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("新建规则编码")
    @XApiPost
    public XServiceResult createRuleNumber(XContext context, CreateRuleNumberInput input) throws ParseException {
        SysDictionaryMapper mapper = context.getBean(SysDictionaryMapper.class);
        QueryWrapper<SysDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysDictionaryEnt::getAlias, "rule_number");
        SysDictionaryEnt sysDictionaryEnt = mapper.selectOne(queryWrapper);
        SysDictionaryEnt newEnt = new SysDictionaryEnt();
        newEnt.setPid(sysDictionaryEnt.getId());
        newEnt.setDicName(input.getDicName());
        newEnt.setRemark(input.getRemark());
        newEnt.setEffectTime(new Date());
        newEnt.setEndTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("9999-12-31 00:00:00"));
        newEnt.setDicKey(PinyinUtil.getPinyin(input.getDicName(), "_"));
        newEnt.updateCreateInfo(context);
        mapper.insert(newEnt);
        SysDictionaryCache.deleteCache(context, "rule_number");
        return XServiceResult.OK;
    }

    @XText("查询规则编码变量")
    @XApiPost
    public XListResult<QuerySysDictionaryViewOutput> queryRuleNumberVars(XContext context) {
        CreateSysDictionaryInput input = new CreateSysDictionaryInput();
        input.setAlias("CODE_GENERATOR_PARAM");
        return querySysDictionarysByAlias(context, input);
    }
}