package pps.core.system.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysDictionaryEnt;
import pps.core.system.entity.SysDictionaryView;
import pps.core.system.entity.SysOrganizationDictionaryEnt;
import pps.core.system.entity.SysOrganizationDictionaryView;
import pps.core.system.mapper.SysDictionaryMapper;
import pps.core.system.mapper.SysDictionaryViewMapper;
import pps.core.system.mapper.SysOrganizationDictionaryMapper;
import pps.core.system.mapper.SysOrganizationDictionaryViewMapper;
import pps.core.system.service.data.sys_organization_dictionary.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("组织机构字典接口")
@XService
public class SysOrganizationDictionaryService {

    @XText("创建组织机构字典")
    @XApiPost
    public XServiceResult createSysOrganizationDictionary(XContext context, CreateSysOrganizationDictionaryInput input) {
        return XTransactionHelper.begin(context, () -> {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
            SysDictionaryMapper dicMapper = context.getBean(SysDictionaryMapper.class);
            SimpleDateFormat sm = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date endTime = null;
            try {
                endTime = sm.parse("9990-12-31 23:59:59");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            QueryWrapper<SysOrganizationDictionaryEnt> query = new QueryWrapper();
            query.lambda().eq(SysOrganizationDictionaryEnt::getOuId, input.getOuId()).gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
            List<SysOrganizationDictionaryEnt> list = mapper.selectList(query);
            /*List<SysOrganizationDictionaryEnt> delList = new ArrayList<>();
            List<SysDictionaryView> dataList = new ArrayList<>();
            if (input.getItems() != null && input.getItems().size() > 0){
                List<SysDictionaryView> items = input.getItems();
                if (ObjectUtils.isNotEmpty(list)) {
                    for (SysOrganizationDictionaryEnt obj : list) {
                        boolean flag = true;
                        for (SysDictionaryView view : items) {
                            if(StringUtils.equals(view.getUid(),obj.getId()) || StringUtils.isEmpty(view.getUid())){
                                flag = false;
                                dataList.add(view);
                            }
                        }
                        if(flag){
                            delList.add(obj);
                        }
                    }
                }else{
                    dataList = input.getItems();
                }
            }*/
            for (SysOrganizationDictionaryEnt obj : list) {
                obj.setEndTime(new Date());
                obj.setModifyById(session.getId());
                obj.setModifyByName(session.getLoginName());
                obj.setModifyTime(new Date());
                mapper.updateById(obj);
            }
            List<SysDictionaryView> dataList = input.getItems();
            if (dataList != null && dataList.size() > 0) {
                for (SysDictionaryView view : dataList) {
                    SysOrganizationDictionaryEnt entity = new SysOrganizationDictionaryEnt();
                    entity.setOuId(input.getOuId());
                    entity.setParentDicCode(view.getPid());
                    entity.setDicCode(view.getDicKey());
                    entity.setEffectTime(new Date());
                    entity.setEndTime(endTime);
                    entity.setCreateById(session.getId());
                    entity.setCreateByName(session.getLoginName());
                    entity.setCreateTime(new Date());
                    entity.setUseLowerLevel(view.getUseLowerLevel());
                    entity.setId(UUID.randomUUID().toString());
                    mapper.insert(entity);
                    if (StringUtils.isNotBlank(view.getPid())) {
                        SysDictionaryEnt dictionaryEnt = new SysDictionaryEnt();
                        dictionaryEnt.setId(view.getPid());
                        dictionaryEnt.setIsOrg(1);
                        dicMapper.updateById(dictionaryEnt);
                    }
                }
            }
            return XServiceResult.OK;
        });
    }

    @XText("修改组织机构字典")
    @XApiPost
    public XServiceResult updateSysOrganizationDictionary(XContext context, UpdateSysOrganizationDictionaryInput input) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        QueryWrapper<SysOrganizationDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationDictionaryEnt::getId, input.getId());
        SysOrganizationDictionaryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除组织机构字典")
    @XApiPost
    public XServiceResult deleteSysOrganizationDictionary(XContext context, DeleteSysOrganizationDictionaryInput input) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        SysDictionaryMapper dicMapper = context.getBean(SysDictionaryMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysDictionaryViewMapper dicViewMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView view = new SysDictionaryView();
        view.setDicPath(input.getId());
        List<SysDictionaryView> sysDictionaryViews = dicViewMapper.selectListsByDicPathIncludeSelf(view);
        List<String> dicIds = sysDictionaryViews.stream().map(p ->
                p.getId()
        ).distinct().collect(Collectors.toList());
        QueryWrapper<SysOrganizationDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(SysOrganizationDictionaryEnt::getDicCode, dicIds);
        if (StringUtils.isNotBlank(input.getOuId())) {
            queryWrapper.lambda().eq(SysOrganizationDictionaryEnt::getOuId, input.getOuId());
        }
        List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = mapper.selectList(queryWrapper);
        if (ObjectUtils.isNotEmpty(sysOrganizationDictionaryEnts)) {
            for (SysOrganizationDictionaryEnt ent : sysOrganizationDictionaryEnts) {
                ent.setEndTime(new Date());
                ent.setModifyTime(new Date());
                ent.setModifyByName(session.getLoginName());
                ent.setModifyById(session.getId());
                mapper.updateById(ent);
            }
        }
        //查询单位属性表是否还有字典，如果没有删除字典表中标记
//        List<SysOrganizationDictionaryEnt> sysDics = mapper.selectList(queryWrapper);
        if (ObjectUtils.isNotEmpty(sysDictionaryViews)) {
            for (SysDictionaryView dicView : sysDictionaryViews) {
                SysDictionaryEnt ent = new SysDictionaryEnt();
                ent.setId(dicView.getId());
                ent.setIsOrg(0);
                ent.setModifyTime(new Date());
                ent.setModifyName(session.getLoginName());
                ent.setModifyUserid(session.getId());
                dicMapper.updateById(ent);
            }
        }
        return XServiceResult.OK;
    }

    @XApiPost
    public XSingleResult<GetSysOrganizationDictionaryOutput> getSysOrganizationDictionary(XContext context, GetSysOrganizationDictionaryInput input) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        QueryWrapper<SysOrganizationDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationDictionaryEnt::getId, input.getId());
        SysOrganizationDictionaryEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysOrganizationDictionaryOutput output = new GetSysOrganizationDictionaryOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("根据组织机构id查询所有字典id")
    @XApiPost
    public XPageResult<QuerySysOrganizationDictionaryOutput> querySysOrganizationDictionary(XContext context, QuerySysOrganizationDictionaryInput input) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        QueryWrapper<SysOrganizationDictionaryEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SysOrganizationDictionaryEnt::getOuId, input.getOuId())
                .gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysOrganizationDictionaryOutput.class);
    }

    @XApiPost
    public XPageResult<QuerySysOrganizationDictionaryOutput> dynamicQuerySysOrganizationDictionary(XContext context, CustomQueryInput input) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        return XMapperHelper.query(mapper, input, SysOrganizationDictionaryEnt.class, QuerySysOrganizationDictionaryOutput.class);
    }

    @XApiPost
    public XSingleResult<GetSysOrganizationDictionaryViewOutput> getSysOrganizationDictionaryView(XContext context, GetSysOrganizationDictionaryViewInput input) {
        SysOrganizationDictionaryViewMapper mapper = context.getBean(SysOrganizationDictionaryViewMapper.class);
        SysOrganizationDictionaryView record = new SysOrganizationDictionaryView();
        XCopyUtils.copyObject(input, record);
        SysOrganizationDictionaryView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysOrganizationDictionaryViewOutput output = new GetSysOrganizationDictionaryViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiPost
    public XPageResult<QuerySysOrganizationDictionaryViewOutput> querySysOrganizationDictionaryView(XContext context, QuerySysOrganizationDictionaryViewInput input) {
        SysOrganizationDictionaryViewMapper mapper = context.getBean(SysOrganizationDictionaryViewMapper.class);
        SysOrganizationDictionaryView record = new SysOrganizationDictionaryView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysOrganizationDictionaryView> list = mapper.selectList(record);
        PageInfo<SysOrganizationDictionaryView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationDictionaryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationDictionaryViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("根据组织机构id查询字典")
    @XApiPost
    public XPageResult<QuerySysDictionaryViewOutput> queryDictionaryViewByOuId(XContext context, QuerySysOrganizationDictionaryViewInput input) {
        //1、根据组织机构id查询字典id（dictCode）
        //2、parentId为1：根据字典code查询所有字典,返回dictPath,整合id列表（把所有字典id拆开放在集合里）,然后id++dicName过滤数据，返回第一级id
        //3、parentId不为1：根据pid+dicName查询下级数据
        SysDictionaryMapper dictionaryMapper = context.getBean(SysDictionaryMapper.class);
        SysDictionaryViewMapper dictionaryViewMapper = context.getBean(SysDictionaryViewMapper.class);
        List<QuerySysDictionaryViewOutput> outputs = new ArrayList<>();
        List<SysDictionaryView> dictionaryViews;
        Integer dictionaryItems;
        PageInfo<SysDictionaryView> pageInfo = new PageInfo<>();
        pageInfo.setPageNum(input.getPage());
        pageInfo.setPageSize(input.getLimit());
        DateTime date = DateUtil.date();
        if (StringUtils.equals(input.getParentId(), "1")) {
            SysDictionaryView recordView = XCopyUtils.copyNewObject(input, SysDictionaryView.class);
            int start = (input.getPage() - 1) * input.getLimit();
            recordView.setStartRow(start);
            recordView.setEndRow(input.getLimit());
            recordView.setEndTime(DateUtil.offsetMinute(date, 1));
            dictionaryViews = dictionaryViewMapper.selectListByConditionByPage(recordView);
            dictionaryItems = dictionaryViewMapper.selectListByConditionByCount(recordView);
            pageInfo.setTotal(dictionaryItems);
        } else {
            SysDictionaryView recordView = XCopyUtils.copyNewObject(input, SysDictionaryView.class);
            recordView.setPid(input.getParentId());
            recordView.setEndTimeStr(date.toString());
            dictionaryViews = dictionaryViewMapper.selectChildListByCondition(recordView);
        }
        if (ObjectUtils.isNotEmpty(dictionaryViews)) {
            List<String> pidList = dictionaryViews.stream().map(p ->
                    p.getId()
            ).distinct().collect(Collectors.toList());
            QueryWrapper<SysDictionaryEnt> queryDict = new QueryWrapper();
            queryDict.lambda().in(SysDictionaryEnt::getPid, pidList);
            List<SysDictionaryEnt> sysDictionaryList = dictionaryMapper.selectList(queryDict);
            pageInfo.setList(dictionaryViews);
            outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysDictionaryViewOutput.class);
            int i = 0;
            for (QuerySysDictionaryViewOutput out : outputs) {
                if (sysDictionaryList != null && sysDictionaryList.size() > 0) {
                    boolean flag = false;
                    for (SysDictionaryEnt obj : sysDictionaryList) {
                        if (StringUtils.equals(out.getId(), obj.getPid())) {
                            flag = true;
                            break;
                        }
                    }
                    if (!flag) {
                        //添加子节点是否打开的标志
                        out.setChildren(null);
                    }
                } else {
                    //添加子节点是否打开的标志
                    out.setChildren(null);
                }
                if (!StringUtils.equals(input.getParentId(), "1")) {
                    out.setChildren(null);
                }
                out.setOrderId(i);
                i++;
            }
            return XPageResult.success(outputs, input, pageInfo.getTotal());
        } else {
            return XPageResult.success(outputs, input, 0);
        }
    }

    public XListResult<SysOrganizationDictionaryEnt> querySysOrganizationDictionarys(XContext context, String ouId, String pid) {
        SysOrganizationDictionaryMapper mapper = context.getBean(SysOrganizationDictionaryMapper.class);
        QueryWrapper<SysOrganizationDictionaryEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(ouId)) {
            queryWrapper.lambda().like(SysOrganizationDictionaryEnt::getOuId, ouId);
        }
        if (StringUtils.isNotBlank(pid)) {
            queryWrapper.lambda().like(SysOrganizationDictionaryEnt::getParentDicCode, pid);
        }
        queryWrapper.lambda().gt(SysOrganizationDictionaryEnt::getEndTime, new Date());
        List<SysOrganizationDictionaryEnt> sysOrganizationDictionaryEnts = mapper.selectList(queryWrapper);
        return XListResult.success(sysOrganizationDictionaryEnts);
    }
}