package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_dictionary.CreateSysDictionaryInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysOrganizationPropertyEnt;
import pps.core.system.entity.SysOrganizationPropertyView;
import pps.core.system.mapper.SysOrganizationPropertyMapper;
import pps.core.system.mapper.SysOrganizationPropertyViewMapper;
import pps.core.system.service.data.sys_organization_property.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@XService
@XText("组织机构属性管理")
public class SysOrganizationPropertyService {

    @XText("查询组织机构介质类型")
    @XApiPost
    public XListResult<GetSysOrganizationPropertyOutput> getSysOrganizationProperty(XContext context, GetSysOrganizationPropertyInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        QueryWrapper<SysOrganizationPropertyEnt> queryWrapper = new QueryWrapper<>();
        Date date = new Date();
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        List<QuerySysDictionaryViewOutput> mediaTypesResult = mediaTypes.getResult();
        if (XStringUtils.isEmpty(input.getOuId())) {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (BusinessConstant.ZERO.equals(session.getIsSuperAdmin())) {
                queryWrapper.lambda().eq(SysOrganizationPropertyEnt::getOuId, session.getOuId())
                        .le(true, SysOrganizationPropertyEnt::getEffectTime, date)
                        .ge(true, SysOrganizationPropertyEnt::getEndTime, date);
            } else {
                List<GetSysOrganizationPropertyOutput> list = new ArrayList<>();
                for (QuerySysDictionaryViewOutput viewOutput : mediaTypesResult) {
                    GetSysOrganizationPropertyOutput output = new GetSysOrganizationPropertyOutput();
                    output.setPropertyValue(viewOutput.getDicKey());
                    output.setPropertyValueName(viewOutput.getDicName());
                    list.add(output);
                }
                return XListResult.success(list);
            }
        } else {
            queryWrapper.lambda().eq(SysOrganizationPropertyEnt::getOuId, input.getOuId())
                    .le(true, SysOrganizationPropertyEnt::getEffectTime, date)
                    .ge(true, SysOrganizationPropertyEnt::getEndTime, date);
        }
        List<SysOrganizationPropertyEnt> propertyEnts = mapper.selectList(queryWrapper);
        List<GetSysOrganizationPropertyOutput> outputs = XCopyUtils.copyNewList(propertyEnts, GetSysOrganizationPropertyOutput.class);
        for (GetSysOrganizationPropertyOutput output : outputs) {
            for (QuerySysDictionaryViewOutput viewOutput : mediaTypesResult) {
                if (viewOutput.getDicKey().equals(output.getPropertyValue())) {
                    output.setPropertyValueName(viewOutput.getDicName());
                }
            }
        }
        return XListResult.success(outputs);
    }

    @XApiPost
    public XListResult<QuerySysOrganizationPropertyOutput> query(XContext context, QuerySysOrganizationPropertyInput input) {
        //普通角色只查询所属企业的介质类型
        PageHelper.startPage(input.getPage(), input.getLimit());
        SysOrganizationPropertyViewMapper mapper = context.getBean(SysOrganizationPropertyViewMapper.class);
        SysOrganizationPropertyView propertyView = new SysOrganizationPropertyView();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        propertyView.setOuId(session.getOuId());
        propertyView.setEndTime(new Date());
        List<SysOrganizationPropertyView> propertyViews = mapper.selectList(propertyView);
        PageInfo<SysOrganizationPropertyView> pageInfo = new PageInfo<>(propertyViews);
        List<QuerySysOrganizationPropertyOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationPropertyOutput.class);
        return XListResult.success(outputs);
    }

    @XApiPost
    public XServiceResult createSysOrganizationProperty(XContext context, CreateSysOrganizationPropertyInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        SysOrganizationPropertyEnt entity = new SysOrganizationPropertyEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateSysOrganizationProperty(XContext context, UpdateSysOrganizationPropertyInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        QueryWrapper<SysOrganizationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationPropertyEnt::getId, input.getId());
        SysOrganizationPropertyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteSysOrganizationProperty(XContext context, DeleteSysOrganizationPropertyInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        QueryWrapper<SysOrganizationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrganizationPropertyEnt::getId, input.getId());
        SysOrganizationPropertyEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XPageResult<QuerySysOrganizationPropertyOutput> querySysOrganizationProperty(XContext context, QuerySysOrganizationPropertyInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        QueryWrapper<SysOrganizationPropertyEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(SysOrganizationPropertyEnt::getId, input.getId())
                .like(SysOrganizationPropertyEnt::getOuId, input.getOuId())
                .like(SysOrganizationPropertyEnt::getPropertyCode, input.getPropertyCode())
                .like(SysOrganizationPropertyEnt::getPropertyValue, input.getPropertyValue())
                .eq(SysOrganizationPropertyEnt::getEffectTime, input.getEffectTime())
                .eq(SysOrganizationPropertyEnt::getEndTime, input.getEndTime())
                .like(SysOrganizationPropertyEnt::getCreateById, input.getCreateById())
                .like(SysOrganizationPropertyEnt::getCreateByName, input.getCreateByName())
                .eq(SysOrganizationPropertyEnt::getCreateTime, input.getCreateTime())
                .like(SysOrganizationPropertyEnt::getModifyById, input.getModifyById())
                .like(SysOrganizationPropertyEnt::getModifyByName, input.getModifyByName())
                .eq(SysOrganizationPropertyEnt::getModifyTime, input.getModifyTime());
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysOrganizationPropertyOutput.class);
    }

    @XApiGet
    public XPageResult<QuerySysOrganizationPropertyOutput> dynamicQuerySysOrganizationProperty(XContext context, CustomQueryInput input) {
        SysOrganizationPropertyMapper mapper = context.getBean(SysOrganizationPropertyMapper.class);
        return XMapperHelper.query(mapper, input, SysOrganizationPropertyEnt.class, QuerySysOrganizationPropertyOutput.class);
    }

    @XApiGet
    public XPageResult<QuerySysOrganizationPropertyViewOutput> querySysOrganizationPropertyView(XContext context, QuerySysOrganizationPropertyViewInput input) {
        SysOrganizationPropertyViewMapper mapper = context.getBean(SysOrganizationPropertyViewMapper.class);
        SysOrganizationPropertyView record = new SysOrganizationPropertyView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysOrganizationPropertyView> list = mapper.selectList(record);
        PageInfo<SysOrganizationPropertyView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationPropertyViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationPropertyViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}