package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysUserWithUserGroupRelEnt;
import pps.core.system.entity.SysUserWithUserGroupRelView;
import pps.core.system.mapper.SysUserWithUserGroupRelMapper;
import pps.core.system.mapper.SysUserWithUserGroupRelViewMapper;
import pps.core.system.service.data.sys_user_with_user_group_rel.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("用户与用户组关系管理")
@XService
public class SysUserWithUserGroupRelService {

    @XText("根据用户查询所绑定的用户组")
    @XApiPost
    public XListResult<QuerySysUserWithUserGroupRelViewOutput> queryUserGroupByUserId(XContext context, QuerySysUserWithUserGroupRelViewInput input) {
        SysUserWithUserGroupRelViewMapper mapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
        SysUserWithUserGroupRelView record = new SysUserWithUserGroupRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserWithUserGroupRelView> list = mapper.selectListByUserId(record);
        List<QuerySysUserWithUserGroupRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserWithUserGroupRelViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("创建用户组和用户关系表")
    @XApiPost
    public XServiceResult createSysUserWithUserGroup(XContext context, CreateSysUserWithUserGroupRelInput input) {
        //SysUserWithUserGroupRel
        SysUserWithUserGroupRelMapper userGroupWithRole = context.getBean(SysUserWithUserGroupRelMapper.class);
        SysUserWithUserGroupRelViewMapper userWithUserGroupRelViewMapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
        //删除原关系
        QueryWrapper<SysUserWithUserGroupRelEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.equals("1", input.getType())) {
            queryWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserGroupId, input.getUserGroupId());
        } else {
            queryWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserId, input.getUserId());
        }
        PageHelper.startPage(1, 1000);
        List<SysUserWithUserGroupRelEnt> sysAuthOuScopeEnts = userGroupWithRole.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(sysAuthOuScopeEnts)) {
            List<String> collect = sysAuthOuScopeEnts.stream().map(SysUserWithUserGroupRelEnt::getId).collect(Collectors.toList());
            userGroupWithRole.deleteBatchIds(collect);
        }
        //创建新关系
        Date now = new Date();
        List<String> groupIdList = input.getUserGroupIdList();
        List<SysUserWithUserGroupRelView> viewList = new ArrayList<>();
        for (String groupId : groupIdList) {
            SysUserWithUserGroupRelView entity = new SysUserWithUserGroupRelView();
            if (StringUtils.equals("1", input.getType())) {
                entity.setUserId(groupId);
                entity.setUserGroupId(input.getUserGroupId());
            } else {
                entity.setUserId(input.getUserId());
                entity.setUserGroupId(groupId);
            }
            entity.setId(UUID.randomUUID().toString());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setCreateById(session.getId());
            entity.setCreateByName(session.getUserName());
            entity.setCreateTime(now);
            viewList.add(entity);
        }
        if (!CollectionUtils.isEmpty(viewList)) {
            userWithUserGroupRelViewMapper.batchInsert(viewList);
        }
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult createSysUserWithUserGroupRel(XContext context, CreateSysUserWithUserGroupRelInput input) {
        //删除原关系
        SysUserWithUserGroupRelMapper delMapper = context.getBean(SysUserWithUserGroupRelMapper.class);
        QueryWrapper<SysUserWithUserGroupRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserId, input.getUserId());
        PageHelper.startPage(1, 1000);
        List<SysUserWithUserGroupRelEnt> sysUserGroupEnts = delMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(sysUserGroupEnts)) {
            List<String> collect = sysUserGroupEnts.stream().map(SysUserWithUserGroupRelEnt::getId).collect(Collectors.toList());
            delMapper.deleteBatchIds(collect);
        }
        SysUserWithUserGroupRelMapper mapper = context.getBean(SysUserWithUserGroupRelMapper.class);
        List<SysUserWithUserGroupRelEnt> list = new ArrayList<>();
        for (String userGroupId : input.getUserGroupIdList()) {
            SysUserWithUserGroupRelEnt entity = new SysUserWithUserGroupRelEnt();
            entity.setUserId(input.getUserId());
            entity.setUserGroupId(userGroupId);
            entity.setSort(input.getSort());
            list.add(entity);
        }
        list.forEach(x -> mapper.insert(x));
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult createSysUserGroupWithUserRel(XContext context, CreateSysUserGroupWithUserRelInput input) {
        //删除原关系
        SysUserWithUserGroupRelMapper delMapper = context.getBean(SysUserWithUserGroupRelMapper.class);
        QueryWrapper<SysUserWithUserGroupRelEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserWithUserGroupRelEnt::getUserGroupId, input.getUserGroupId());
        PageHelper.startPage(1, 1000);
        List<SysUserWithUserGroupRelEnt> sysUserGroupEnts = delMapper.selectList(queryWrapper);
        if (!CollectionUtils.isEmpty(sysUserGroupEnts)) {
            List<String> collect = sysUserGroupEnts.stream().map(SysUserWithUserGroupRelEnt::getId).collect(Collectors.toList());
            delMapper.deleteBatchIds(collect);
        }
        SysUserWithUserGroupRelMapper mapper = context.getBean(SysUserWithUserGroupRelMapper.class);
        List<SysUserWithUserGroupRelEnt> list = new ArrayList<>();
        for (String userId : input.getUserIdList()) {
            SysUserWithUserGroupRelEnt entity = new SysUserWithUserGroupRelEnt();
            entity.setUserId(userId);
            entity.setUserGroupId(input.getUserGroupId());
            entity.setSort(input.getSort());
            list.add(entity);
        }
        list.forEach(x -> mapper.insert(x));
        return XServiceResult.OK;
    }

    @XApiPost
    public XListResult<QuerySysUserWithUserGroupRelViewOutput> querySysUserWithUserGroupRelView(XContext context, QuerySysUserWithUserGroupRelViewInput input) {
        SysUserWithUserGroupRelViewMapper mapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
        SysUserWithUserGroupRelView record = new SysUserWithUserGroupRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserWithUserGroupRelView> list = mapper.selectList(record);
        List<QuerySysUserWithUserGroupRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserWithUserGroupRelViewOutput.class);
        return XListResult.success(outputs);
    }

    @XApiPost
    public XListResult<QuerySysUserGroupWithUserRelViewOutput> querySysUserGroupWithUserRelView(XContext context, QuerySysUserGroupWithUserRelViewInput input) {
        SysUserWithUserGroupRelViewMapper mapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
        SysUserWithUserGroupRelView record = new SysUserWithUserGroupRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserWithUserGroupRelView> list = mapper.selectList(record);
        List<QuerySysUserGroupWithUserRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserGroupWithUserRelViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据用户组id查询选中的用户")
    @XApiPost
    public XListResult<QuerySysUserGroupWithUserRelViewOutput> querySysUserGroupWithUserRelViewByUserGroupId(XContext context, QuerySysUserGroupWithUserRelViewInput input) {
        SysUserWithUserGroupRelViewMapper mapper = context.getBean(SysUserWithUserGroupRelViewMapper.class);
        SysUserWithUserGroupRelView record = new SysUserWithUserGroupRelView();
        XCopyUtils.copyObject(input, record);
        List<SysUserWithUserGroupRelView> list = mapper.selectListByUserGroupId(record);
        List<QuerySysUserGroupWithUserRelViewOutput> outputs = XCopyUtils.copyNewList(list, QuerySysUserGroupWithUserRelViewOutput.class);
        return XListResult.success(outputs);
    }
}