package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.sys_area.GetSysAreaInput;
import pps.cloud.system.service.data.sys_area.GetSysAreaOutput;
import pps.cloud.system.service.data.sys_area.QuerySysAreaInput;
import pps.cloud.system.service.data.sys_area.QuerySysAreaOutput;
import pps.core.system.entity.SysAreaEnt;
import pps.core.system.entity.SysAreaView;
import pps.core.system.mapper.SysAreaMapper;
import pps.core.system.mapper.SysAreaViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiPost;

import java.util.List;
import java.util.Objects;

@XService
public class SystemAreaServiceImpl implements SystemAreaService {

    @XApiPost
    @Override
    public XListResult<GetSysAreaOutput> getSysAreaList(XContext context, GetSysAreaInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        List<SysAreaView> sysAreaViews = mapper.selectListAll(new SysAreaView());
        List<GetSysAreaOutput> getSysAreaOutputs = XCopyUtils.copyNewList(sysAreaViews, GetSysAreaOutput.class);
        return XListResult.success(getSysAreaOutputs);
    }

    /**
     * 地区Cloud服务-条件查询地区
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSysAreaOutput}>
     */
    @Override
    public XSingleResult<GetSysAreaOutput> getSysAreaByParam(XContext context, GetSysAreaInput input) {
        Integer id = input.getId();
        String name = input.getName();
        Integer parentId = input.getParentId();
        SysAreaMapper mapper = context.getBean(SysAreaMapper.class);
        SysAreaEnt sysAreaEnt = mapper.selectOne(new LambdaQueryWrapper<SysAreaEnt>()
                .eq(Objects.nonNull(id), SysAreaEnt::getId, id)
                .eq(Objects.nonNull(parentId), SysAreaEnt::getParentId, parentId)
                .eq(StringUtils.isNotEmpty(name), SysAreaEnt::getName, name)
                .last("LIMIT 1")
        );
        return XSingleResult.success(XCopyUtils.copyNewObject(sysAreaEnt, GetSysAreaOutput.class));
    }

    /**
     * 地区Cloud服务-查询地区路径
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSysAreaOutput}>
     */
    @Override
    public XSingleResult<GetSysAreaOutput> getSysAreaPath(XContext context, GetSysAreaInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        SysAreaView sysAreaView = mapper.selectSysAreaPath(input.getId());
        return XSingleResult.success(XCopyUtils.copyNewObject(sysAreaView, GetSysAreaOutput.class));
    }

    /**
     * 获取省和市
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link QuerySysAreaOutput}>
     */
    @Override
    public XListResult<QuerySysAreaOutput> getProvinceAndCity(XContext context, QuerySysAreaInput input) {
        SysAreaViewMapper mapper = context.getBean(SysAreaViewMapper.class);
        List<SysAreaView> viewList = mapper.selectProvinceAndCity(input.getProvinceList());
        return XListResult.success(XCopyUtils.copyNewList(viewList, QuerySysAreaOutput.class));
    }
}