package pps.core.system.utils;

import org.apache.commons.lang3.StringUtils;
import pps.core.common.utils.PatternUtil;
import pps.core.common.utils.RSAUtil;
import pps.core.system.config.LoginConfig;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XServiceContext;
import xstartup.base.util.XEncryptUtils;

/**
 * @author lixueyan
 * @date 2023/3/8 0008 10:45
 */
@XText("用户信息处理工具类")
public class UserInfoUtil {

    //敏感信息rsa加密
    public static String encrypt(String str) {
        if (StringUtils.isBlank(str)) {
            return str;
        }
        XContext context = XServiceContext.build(UserInfoUtil.class);
        String publicKeyValue = LoginConfig.userRsaPublicKey.findValue(context);
        publicKeyValue = XEncryptUtils.decrypt(publicKeyValue);
        return RSAUtil.rsaEncrypt(str, publicKeyValue);
    }

    //敏感信息rsa解密
    public static String decrypt(String str) {
        if (StringUtils.isBlank(str)) {
            return str;
        }
        XContext context = XServiceContext.build(UserInfoUtil.class);
        try {
            String privateKeyValue = LoginConfig.userRsaPrivateKey.findValue(context);
            privateKeyValue = XEncryptUtils.decrypt(privateKeyValue);
            return RSAUtil.RsaDecrypt(str, privateKeyValue);
        } catch (Exception e) {
            context.getLogger().error("敏感信息解密失败， 使用原信息");
            return str;
        }
    }

    //敏感信息解密并脱敏
    public static String desensitization(String str) {
        return PatternUtil.desensitization(decrypt(str));
    }

}
