package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BasePowerLineDieselEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BaseDieselGeneratorMapper;
import pps.core.base.mapper.BasePowerLineDieselMapper;
import pps.core.base.service.data.base_diesel_generator.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Objects;

/**
 * 柴发设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseDieselGeneratorService {

    /**
     * 柴发设备模块--新增
     * POST /base/base-diesel-generator/create-base-diesel-generator
     * 接口ID：105601993
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601993
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("柴发设备模块--新增")
    @XApiPost
    public XServiceResult createBaseDieselGenerator(XContext context, CreateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        if (this.checkMakerNumberExists(mapper, input.getMakerNumber(), null)) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BaseDieselGeneratorEnt entity = XCopyUtils.copyNewObject(input, BaseDieselGeneratorEnt.class);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 柴发设备模块--修改
     * POST /base/base-diesel-generator/update-base-diesel-generator
     * 接口ID：105601994
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601994
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("柴发设备模块--修改")
    @XApiPost
    public XServiceResult updateBaseDieselGenerator(XContext context, UpdateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        if (this.checkMakerNumberExists(mapper, input.getMakerNumber(), input.getId())) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BaseDieselGeneratorEnt entity = this.selectOneByDieselId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 柴发设备模块--删除
     * POST /base/base-diesel-generator/delete-base-diesel-generator
     * 接口ID：105601998
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601998
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("柴发设备模块--删除")
    @XApiPost
    public XServiceResult deleteBaseDieselGenerator(XContext context, DeleteBaseDieselGeneratorInput input) {
        String dieselId = input.getId();
        if (this.checkDieselIsReference(context, dieselId)) {
            return XServiceResult.error(context, BusinessError.DieselReference);
        }
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        if (Objects.isNull(this.selectOneByDieselId(dieselId, mapper))) {
            return XServiceResult.error(context, XError.NotFound);
        }
        BaseDieselGeneratorEnt entity = new BaseDieselGeneratorEnt();
        entity.setId(dieselId);
        entity.setIsDeleted(BusinessConstant.ZERO);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 柴发设备模块--详情
     * GET /base/base-diesel-generator/get-base-diesel-generator
     * 接口ID：105601991
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601991
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBaseDieselGeneratorOutput}>
     */
    @XText("柴发设备模块--详情")
    @XApiGet
    public XSingleResult<GetBaseDieselGeneratorOutput> getBaseDieselGenerator(XContext context, GetBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        BaseDieselGeneratorEnt entity = this.selectOneByDieselId(input.getId(), mapper);
        if (Objects.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDieselGeneratorOutput output = XCopyUtils.copyNewObject(entity, GetBaseDieselGeneratorOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 柴发设备模块--分页查询
     * GET /base/base-diesel-generator/query-base-diesel-generator
     * 接口ID：105601992
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105601992
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBaseDieselGeneratorOutput}>
     */
    @XText("柴发设备模块--分页查询")
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorOutput> queryBaseDieselGenerator(XContext context, QueryBaseDieselGeneratorInput input) {
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists(!CharSequenceUtil.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId()), "SELECT 1 FROM base_power_line_diesel w WHERE base_diesel_generator.id = w.diesel_id AND w.is_deleted = 1 AND w.line_id <> {0}", input.getLineId())
                    .notExists(CharSequenceUtil.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId()), "SELECT 1 FROM base_power_line_diesel w WHERE base_diesel_generator.id = w.diesel_id AND w.is_deleted = 1")
                    .eq(BaseDieselGeneratorEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BaseDieselGeneratorEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .like(CharSequenceUtil.isNotEmpty(ouName), BaseDieselGeneratorEnt::getOuName, ouName)
                .like(CharSequenceUtil.isNotEmpty(deviceName), BaseDieselGeneratorEnt::getDeviceName, deviceName)
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE)
                .orderByAsc(BaseDieselGeneratorEnt::getOuName, BaseDieselGeneratorEnt::getDeviceName);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDieselGeneratorOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查柴发设备是否被引用
     *
     * @param context  上下文
     * @param dieselId 柴发设备ID
     * @return boolean
     */
    private boolean checkDieselIsReference(XContext context, String dieselId) {
        BasePowerLineDieselMapper mapper = context.getBean(BasePowerLineDieselMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineDieselEnt>()
                .eq(BasePowerLineDieselEnt::getDieselId, dieselId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过设备ID查询设备信息
     *
     * @param dieselId 设备id
     * @param mapper   映射器
     * @return {@link BaseDieselGeneratorEnt}
     */
    private BaseDieselGeneratorEnt selectOneByDieselId(String dieselId, BaseDieselGeneratorMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseDieselGeneratorEnt>()
                .eq(BaseModel::getId, dieselId)
        );
    }

    /**
     * 检查编号是否存在
     *
     * @param mapper      映射器
     * @param makerNumber 出场编号
     * @param id          id
     * @return boolean
     */
    private boolean checkMakerNumberExists(BaseDieselGeneratorMapper mapper, String makerNumber, String id) {
        Long count = mapper.selectCount(new LambdaQueryWrapper<BaseDieselGeneratorEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BaseDieselGeneratorEnt::getMakerNumber, makerNumber)
                .ne(CharSequenceUtil.isNotEmpty(id), BaseModel::getId, id)
        );
        return count > 0;
    }
}