package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.SysPageInfoEnt;
import pps.core.base.mapper.SysPageInfoMapper;
import pps.core.base.service.data.sys_page_info.*;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.Date;
import java.util.List;
import java.util.UUID;

@XService
public class SysPageInfoService {

    @XText("创建页面配置")
    @XApiPost
    public XServiceResult createSysPageInfo(XContext context, CreateSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysPageInfoEnt entity = new SysPageInfoEnt();
        XCopyUtils.copyObject(input, entity);
        entity.setId(UUID.randomUUID().toString());
        entity.setPageId(UUID.randomUUID().toString());
        entity.setReleaseState(1);
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getLoginName());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("修改页面配置")
    @XApiPost
    public XServiceResult updateSysPageInfo(XContext context, UpdateSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPageInfoEnt::getId, input.getId());
        SysPageInfoEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        if (StringUtils.isNotBlank(input.getPageContext())) {
            entity.setPageContext(input.getPageContext());
        }
        if (StringUtils.isNotBlank(input.getPageName())) {
            entity.setPageName(input.getPageName());
        }
        if (StringUtils.isNotBlank(input.getPageRemark())) {
            entity.setPageRemark(input.getPageRemark());
        }
        entity.setModifyTime(new Date());
        entity.setModifyByName(session.getLoginName());
        entity.setModifyById(session.getId());
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除页面配置")
    @XApiPost
    public XServiceResult deleteSysPageInfo(XContext context, DeleteSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPageInfoEnt::getId, input.getId());
        SysPageInfoEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XText("获取详情")
    @XApiGet
    public XSingleResult<GetSysPageInfoOutput> getSysPageInfo(XContext context, GetSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPageInfoEnt::getId, input.getId());
        SysPageInfoEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysPageInfoOutput output = new GetSysPageInfoOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("条件查询列表数据")
    @XApiGet
    public XPageResult<QuerySysPageInfoOutput> querySysPageInfo(XContext context, QuerySysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(input.getPageId())) {
            queryWrapper.lambda().eq(SysPageInfoEnt::getPageId, input.getPageId());
        }
        if (StringUtils.isNotBlank(input.getPageName())) {
            queryWrapper.lambda().like(SysPageInfoEnt::getPageName, input.getPageName());
        }
        if (ObjectUtils.isNotEmpty(input.getReleaseState())) {
            queryWrapper.lambda().eq(SysPageInfoEnt::getReleaseState, input.getReleaseState());
        }
        queryWrapper.lambda().orderByDesc(SysPageInfoEnt::getCreateTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QuerySysPageInfoOutput.class);
    }

    @XText("发布页面信息")
    @XApiGet
    public XServiceResult releasePageInfo(XContext context, GetSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        //根据id查询需要发布的数据
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPageInfoEnt::getId, input.getId());
        SysPageInfoEnt sysPageInfoEnt = mapper.selectOne(queryWrapper);
        //查询当前页面处于发布状态的版本删除
        QueryWrapper<SysPageInfoEnt> queryStateVersion = new QueryWrapper();
        queryStateVersion.lambda().eq(SysPageInfoEnt::getPageId, sysPageInfoEnt.getPageId())
                .eq(SysPageInfoEnt::getReleaseVersion, 1);
        List<SysPageInfoEnt> sysPageInfos = mapper.selectList(queryStateVersion);
        if (ObjectUtils.isNotEmpty(sysPageInfos)) {
            SysPageInfoEnt pageInfoEnt = sysPageInfos.get(0);
            mapper.deleteById(pageInfoEnt);
        }
        //查询发布的最大版本号
        QueryWrapper<SysPageInfoEnt> queryMaxVersion = new QueryWrapper();
        queryMaxVersion.lambda().eq(SysPageInfoEnt::getPageId, sysPageInfoEnt.getPageId()).isNotNull(SysPageInfoEnt::getReleaseVersion)
                .orderByDesc(SysPageInfoEnt::getReleaseVersion);
        List<SysPageInfoEnt> sysPageInfoEnts = mapper.selectList(queryMaxVersion);
        if (ObjectUtils.isNotEmpty(sysPageInfoEnts)) {
            SysPageInfoEnt sysPageInfo = sysPageInfoEnts.get(0);
            if (ObjectUtils.isNotEmpty(sysPageInfo.getReleaseVersion())) {
                Integer version = sysPageInfo.getReleaseVersion() + 1;
                sysPageInfoEnt.setReleaseVersion(version);
            } else {
                sysPageInfoEnt.setReleaseVersion(1);
            }
        } else {
            sysPageInfoEnt.setReleaseVersion(1);
        }
        sysPageInfoEnt.setReleaseState(1);
        sysPageInfoEnt.setReleaseTime(new Date());
        mapper.updateById(sysPageInfoEnt);
        return XServiceResult.OK;
    }

    @XText("根据pageId获取详情")
    @XApiGet
    public XSingleResult<GetSysPageInfoOutput> getSysPageInfoByPageId(XContext context, GetSysPageInfoInput input) {
        SysPageInfoMapper mapper = context.getBean(SysPageInfoMapper.class);
        QueryWrapper<SysPageInfoEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysPageInfoEnt::getPageId, input.getPageId())
                .eq(SysPageInfoEnt::getReleaseState, 1);
        SysPageInfoEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetSysPageInfoOutput output = new GetSysPageInfoOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }
}