package pps.cloud.space.service;

import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendInput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendOutput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyAverageInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

/**
 * 日用电趋势计算Cloud模块
 *
 * @author ZWT
 * @date 2024/05/22 17:15
 */
@XService
@XText("日用电趋势计算Cloud模块")
public interface IDailyElectricityTrendCloudService {

    /**
     * 日用电趋势计算Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @XText("日用电趋势计算Cloud模块--定时任务")
    XServiceResult dailyElectricityTrendJob(XContext context);

    /**
     * 用电趋势统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetLineDailyElectricityTrendOutput }>
     */
    @XText("日用电趋势计算Cloud模块--用电趋势统计")
    XListResult<GetLineDailyElectricityTrendOutput> queryDailyElectricityTrend(XContext context, GetLineDailyElectricityTrendInput input);

    /**
     * 累积用电
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetLineDailyElectricityTrendOutput }>
     */
    @XText("日用电趋势计算Cloud模块--累积用电")
    XSingleResult<GetLineDailyElectricityTrendOutput> querySumDailyElectricity(XContext context, GetLineDailyElectricityTrendInput input);

    /**
     * 井口分析分页
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link GetWellDailyElectricityTrendOutput }>
     */
    @XText("日用电趋势计算Cloud模块--井口分析分页")
    XPageResult<GetWellDailyElectricityTrendOutput> queryWellheadAnalysisPage(XContext context, GetWellDailyElectricityTrendInput input);

    /**
     * 查询井日平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellDailyElectricityTrendOutput }>
     */
    @XText("日用电趋势计算Cloud模块--井日平均值")
    XSingleResult<GetWellDailyElectricityTrendOutput> queryWellDailyAverage(XContext context, GetWellDailyAverageInput input);
}
