package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.space.entity.*;
import pps.core.space.mapper.*;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadInput;
import pps.core.space.service.data.space_optimize_short_wellhead.QuerySpaceOptimizeShortWellheadOutput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 间开优化对外提供接口服务
 *
 * @author ZWT
 * @date 2024/04/08 14:21
 */
@XService
@XText("间开优化对外提供接口服务")
public class SpaceOptimizeApiService extends SpaceOptimizeBaseService {

    @XApiGet(anonymous = true)
    @XText("极短期间开优化--计算一个率")
    public XServiceResult getOptimization(XContext context) {
        DateTime of = DateTime.of("2024-04-02", "yyyy-MM-dd");
        //先查并网线路
        IBasePowerLineCloudService lineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineOutputXListResult = lineCloudService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .gridTypeKey("1")
                .build());
        //市电峰谷
        IBasePriceStrategyCloudService strategyCloudService = context.getBean(IBasePriceStrategyCloudService.class);
        SpaceInstitutionDetailMapper detailMapper = context.getBean(SpaceInstitutionDetailMapper.class);
        SpaceInstitutionWellheadMapper wellheadMapper = context.getBean(SpaceInstitutionWellheadMapper.class);
        SpaceOptimizeShortDurationMapper shortDurationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        SpaceInstitutionDetailEnt detailEnt;
        List<SpaceInstitutionWellheadEnt> wellheadList;
        XListResult<DynamicQueryBasePowerLinePlantViewOutput> plantResult;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList;
        XListResult<GetBasePriceStrategyDetailOutput> strategyResult;
        List<GetBasePriceStrategyDetailOutput> strategyList;
        List<SpaceOptimizeShortDurationEnt> shortDurationList;
        List<DynamicQueryPlantPredictedPowerOutput> generationHourList;
        List<DynamicQueryBasePowerLineOutput> lineList = lineOutputXListResult.getResult();
        //分母(谷电和光伏发电去重总时间)
        long lowAndPowerSumMinute = 0;
        //分子已优化(谷电和光伏发电去重总时间内间开时间)
        long optimizeSumMinute = 0;
        //分子未优化(谷电和光伏发电去重总时间内间开时间)
        long unOptimizeSumMinute = 0;
        for (DynamicQueryBasePowerLineOutput line : lineList) {
            //查对应间开制度
            detailEnt = detailMapper.selectOne(new LambdaQueryWrapper<SpaceInstitutionDetailEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionDetailEnt::getIsCurrentBasic, BusinessConstant.ZERO)
                    .eq(SpaceInstitutionDetailEnt::getLineId, line.getId())
            );
            if (ObjectUtil.isEmpty(detailEnt)) {
                continue;
            }
            //查间开关联井口(间抽井,小间开)
            wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceInstitutionWellheadEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceInstitutionWellheadEnt::getInstitutionId, detailEnt.getId())
                    .eq(SpaceInstitutionWellheadEnt::getRunTypeKey, BusinessConstant.INTERVAL_PUMPING_WELL)
                    .eq(SpaceInstitutionWellheadEnt::getIntervalTypeKey, "0")
                    .orderByAsc(SpaceInstitutionWellheadEnt::getStartSeq)
            );
            if (CollUtil.isEmpty(wellheadList)) {
                continue;
            }
            //查线路下所有电站
            plantResult = lineCloudService.getPowerLinePlantViewList(context, DynamicQueryBasePowerLinePlantInput.builder()
                    .lineId(line.getId())
                    .build());
            plantList = plantResult.getResult();
            if (CollUtil.isEmpty(plantList)) {
                continue;
            }
            //查线路关联市电峰谷
            strategyResult = strategyCloudService.queryStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                    .lineId(line.getId())
                    .periodTypeKey("LOW")
                    .strategyMonth("4")
                    .build());
            strategyList = strategyResult.getResult();
            if (CollUtil.isEmpty(strategyList)) {
                continue;
            }
            //查间开时间段
            shortDurationList = shortDurationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, of)
                    .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadList.stream()
                            .map(SpaceInstitutionWellheadEnt::getWellheadId)
                            .collect(Collectors.toList())
                    )
                    .orderByAsc(SpaceOptimizeShortDurationEnt::getOpenWellTime)
            );
            if (CollUtil.isEmpty(shortDurationList)) {
                continue;
            }
            //查当日预测发电时间段
            generationHourList = this.getAveragePowerGenerationListByPlantIds(context, DynamicQueryPlantPredictedPowerInput.builder()
                    .plantIds(plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getPlantId)
                            .collect(Collectors.toList()))
                    .startTime(of.toString())
                    .endTime(DateUtil.offsetDay(of, 1).toString())
                    .dateType(BusinessConstant.ONE)
                    .build());
            //取有发电量的时间段
            generationHourList = generationHourList.stream()
                    .filter(generationHour -> generationHour.getPower().compareTo(BigDecimal.ZERO) > 0)
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(generationHourList)) {
                continue;
            }
            List<SpaceOptimizeDurationDTO> powerSpaceList = new ArrayList<>(4);
            DateTime beginDate = DateUtil.date(generationHourList.get(0).getCreateTime());
            DateTime endDate = DateUtil.date(generationHourList.get(generationHourList.size() - 1).getCreateTime());
            powerSpaceList.add(
                    SpaceOptimizeDurationDTO.builder()
                            .openTime(beginDate)
                            .closeTime(endDate)
                            .build()
            );
            //计算发电时间(分钟)
            long powerMinute = DateUtil.between(beginDate, endDate, DateUnit.MINUTE);
            for (GetBasePriceStrategyDetailOutput strategyDetail : strategyList) {
                DateTime startTime = DateUtil.parse(strategyDetail.getStartTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                DateTime endTime;
                if (CharSequenceUtil.equals(strategyDetail.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                    endTime = BusinessConstant.DATE_FLAG;
                } else {
                    endTime = DateUtil.parse(strategyDetail.getEndTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                }
                //判断
                if (startTime.compareTo(beginDate) <= 0 && endTime.compareTo(beginDate) >= 0) {
                    endTime = beginDate;
                } else if (startTime.compareTo(endDate) <= 0 && endTime.compareTo(endDate) >= 0) {
                    startTime = endDate;
                } else if (startTime.compareTo(beginDate) >= 0 && endTime.compareTo(endDate) <= 0) {
                    //时间段和发电时间重合,排除
                    continue;
                }
                powerSpaceList.add(
                        SpaceOptimizeDurationDTO.builder()
                                .openTime(startTime)
                                .closeTime(endTime)
                                .build()
                );
                //计算谷电时间(分钟) 累加
                powerMinute += DateUtil.between(startTime, endTime, DateUnit.MINUTE);
            }
            lowAndPowerSumMinute += powerMinute;
            //优化时长(分钟)
            long optimizeMinute = 0;
            //未优化时长(分钟)
            long unOptimizeMinute = 0;
            //计算间开时间
            for (SpaceOptimizeShortDurationEnt duration : shortDurationList) {
                DateTime openWellTime = DateUtil.parse(duration.getOpenWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                DateTime closeWellTime;
                if (CharSequenceUtil.equals(duration.getCloseWellTime(), BusinessConstant.END_OF_DAY_TIME)) {
                    closeWellTime = BusinessConstant.DATE_FLAG;
                } else {
                    closeWellTime = DateUtil.parse(duration.getCloseWellTime() + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
                }
                for (SpaceOptimizeDurationDTO dto : powerSpaceList) {
                    DateTime openTime = DateUtil.date(dto.getOpenTime());
                    DateTime closeTime = DateUtil.date(dto.getCloseTime());
                    if (openWellTime.compareTo(closeTime) >= 0) {
                        continue;
                    } else if (closeWellTime.compareTo(openTime) <= 0) {
                        continue;
                    } else if (openWellTime.compareTo(openTime) <= 0 && closeWellTime.compareTo(closeTime) <= 0) {
                        openWellTime = openTime;
                    } else if (openWellTime.compareTo(openTime) < 0 && closeWellTime.compareTo(closeTime) > 0) {
                        openWellTime = openTime;
                        closeWellTime = closeTime;
                    } else if (openWellTime.compareTo(openTime) >= 0 && closeWellTime.compareTo(closeTime) >= 0) {
                        closeWellTime = closeTime;
                    }
                    if (duration.getIsOptimize().equals(BusinessConstant.ZERO)) {
                        optimizeMinute += DateUtil.between(openWellTime, closeWellTime, DateUnit.MINUTE);
                        optimizeSumMinute += optimizeMinute;
                    } else {
                        unOptimizeMinute += DateUtil.between(openWellTime, closeWellTime, DateUnit.MINUTE);
                        unOptimizeSumMinute += unOptimizeMinute;
                    }
                }
            }
        }
        //分母
        BigDecimal denominator = new BigDecimal(lowAndPowerSumMinute);
        BigDecimal optimizeNumerator = new BigDecimal(optimizeSumMinute);
        BigDecimal unOptimizeNumerator = new BigDecimal(unOptimizeSumMinute);
        //计算率
        BigDecimal optimizeDivide = optimizeNumerator.divide(denominator, 2, RoundingMode.HALF_UP);
        BigDecimal unOptimizeDivide = unOptimizeNumerator.divide(denominator, 2, RoundingMode.HALF_UP);
        BigDecimal subtract = optimizeDivide.subtract(unOptimizeDivide);
        return XServiceResult.OK;
    }

    /**
     * 每日间开优化结果
     *
     * @param context 上下文
     * @return {@link XListResult}<{@link QuerySpaceOptimizeShortWellheadOutput}>
     */
    @XApiPost(anonymous = true)
    @XText("极短期间开优化--每日间开优化结果")
    public XListResult<QuerySpaceOptimizeShortWellheadOutput> getDailyOptimization(XContext context, QuerySpaceOptimizeShortWellheadInput input) {
        DateTime now = DateUtil.beginOfDay(DateUtil.date());
        Date queryDate = input.getQueryDate();
        if (Objects.isNull(queryDate)) {
            queryDate = now;
        } else {
            queryDate = DateUtil.beginOfDay(queryDate);
        }
        List<QuerySpaceOptimizeShortWellheadOutput> outputs = new ArrayList<>(32);
        //判断日期大小看取15天还是1天数据
        if (DateUtil.compare(queryDate, now) > 0) {
            //15天
            SpaceOptimizeUltraWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeUltraWellheadMapper.class);
            List<SpaceOptimizeUltraWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeUltraWellheadEnt>()
                    .eq(SpaceOptimizeUltraWellheadEnt::getOptimizeDate, queryDate)
                    .orderByAsc(SpaceOptimizeUltraWellheadEnt::getStartSeq)
            );
            if (CollUtil.isEmpty(wellheadList)) {
                return XListResult.success(Collections.emptyList());
            }
            SpaceOptimizeUltraDurationMapper durationMapper = context.getBean(SpaceOptimizeUltraDurationMapper.class);
            List<SpaceOptimizeUltraDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeUltraDurationEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeUltraDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                    .eq(SpaceOptimizeUltraDurationEnt::getOptimizeDate, queryDate)
                    .in(SpaceOptimizeUltraDurationEnt::getWellheadId, wellheadList.stream()
                            .map(SpaceOptimizeUltraWellheadEnt::getWellheadId)
                            .collect(Collectors.toList())
                    )
                    .orderByAsc(SpaceOptimizeUltraDurationEnt::getOpenWellTime)
            );
            //封装数据
            List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
            Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
            for (SpaceOptimizeUltraWellheadEnt wellheadEnt : wellheadList) {
                outputs.add(
                        QuerySpaceOptimizeShortWellheadOutput.builder()
                                .wellNumber(wellheadEnt.getWellNumber())
                                .optimizeList(ServiceUtil.getAllDateDuration(collect.get(wellheadEnt.getWellheadId())))
                                .build()
                );
            }
        } else {
            //1天
            SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
            List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                    .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, queryDate)
                    .orderByAsc(SpaceOptimizeShortWellheadEnt::getStartSeq)
            );
            if (CollUtil.isEmpty(wellheadList)) {
                return XListResult.success(Collections.emptyList());
            }
            SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
            List<SpaceOptimizeShortDurationEnt> durationList = durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                    .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                    .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, BusinessConstant.ZERO)
                    .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, queryDate)
                    .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadList.stream()
                            .map(SpaceOptimizeShortWellheadEnt::getWellheadId)
                            .collect(Collectors.toList())
                    )
                    .orderByAsc(SpaceOptimizeShortDurationEnt::getOpenWellTime)
            );
            //封装数据
            List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
            Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
            for (SpaceOptimizeShortWellheadEnt wellheadEnt : wellheadList) {
                outputs.add(
                        QuerySpaceOptimizeShortWellheadOutput.builder()
                                .wellNumber(wellheadEnt.getWellNumber())
                                .optimizeList(ServiceUtil.getAllDateDuration(collect.get(wellheadEnt.getWellheadId())))
                                .build()
                );
            }
        }
        return XListResult.success(outputs);
    }
}
