package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.core.auth.HttpRequestClient;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.PredictedPowerView;
import pps.core.prediction.entity.WellheadDailyProductionSituationView;
import pps.core.prediction.mapper.PredictedPowerViewMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationViewMapper;
import pps.core.prediction.service.data.wellhead_daily_production_situation.WellheadDailyProductionSituationJob;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 19:05
 */
@XService
public class PlantPredictedPowerCloudServiceImpl implements IPlantPredictedPowerCloudService {

    /**
     * 光伏预测Cloud模块--条件查询时段发电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List<PredictedPowerView> list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerHalfHour(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List<PredictedPowerView> list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerHalfHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerHalfHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    /**
     * 光伏预测Cloud模块--获取每小时平均发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationHourListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        String plantId = input.getPlantId();
        List<String> plantIds = input.getPlantIds();
        String yearTime = input.getYearTime();
        String monthTime = input.getMonthTime();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    /**
     * 日耗电日产液信息定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult wellTechDailyJob(XContext context) {
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        Map<String, Object> param = new HashMap<>(4);
        param.put("Calc_Date", DateUtil.yesterday().toString("yyyy-MM-dd"));
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_WELL_TECH_DAILY;
        String resultStr = client.doPostForm(url, param);
        context.getLogger().info("*** CQ_WELL_TECH_DAILY : {}", resultStr);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        String data = jsonObject.getString("data");
        List<WellheadDailyProductionSituationJob> list = JSON.parseArray(data, WellheadDailyProductionSituationJob.class);
        if (CollUtil.isEmpty(list)) {
            context.getLogger().info("------ WellTechDailyJob 未发现数据: {}", DateUtil.yesterday().toString("yyyy-MM-dd"));
            return XServiceResult.OK;
        }
        List<String> collect = list.stream()
                .map(WellheadDailyProductionSituationJob::getWellNumber)
                .collect(Collectors.toList());
        IBaseWellheadCloudService bean = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = bean.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .wellNumbers(collect)
                .build());
        result.throwIfFail();
        Map<String, String> map = result.getResult().stream()
                .collect(Collectors.toMap(DynamicQueryBaseWellheadOutput::getWellNumber, DynamicQueryBaseWellheadOutput::getId));
        List<WellheadDailyProductionSituationView> dtoList = new ArrayList<>(list.size());
        for (WellheadDailyProductionSituationJob wellhead : list) {
            dtoList.add(WellheadDailyProductionSituationView.builder()
                    .wellNumber(wellhead.getWellNumber())
                    .dataDate(wellhead.getDataDate())
                    .dailyLiquidProduction(wellhead.getDailyLiquidProduction())
                    .productionTime(wellhead.getProductionTime())
                    .dailyElectricityConsumption(wellhead.getDailyElectricityConsumption())
                    .wellheadId(map.get(wellhead.getWellNumber()))
                    .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            WellheadDailyProductionSituationViewMapper mapper = context.getBean(WellheadDailyProductionSituationViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }
}
