package pps.core.prediction.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.DoPlantPredictedPowerDataInput;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataInput;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataOutput;
import pps.core.prediction.utils.HttpUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XDateUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


@XService
@XText("电站预测发电数据表")
public class PlantPredictedPowerDataService {

    @XApiGet
    @XText("根据电站预测4小时内的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getFourPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        if (XStringUtils.isEmpty(input.getStartTime())) {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, date);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addHours(date, 4));
        } else {
            String dataDate = XDateUtils.getDateString(new Date()) + " " + input.getStartTime() + ":00";
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, dataDate);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addHours(XDateUtils.parse(dataDate), 4));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item -> {
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测7天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getSevenPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        if (XStringUtils.isEmpty(input.getStartTime())) {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, date);
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, 7));
        } else {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, input.getStartTime());
            queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(XDateUtils.parse(input.getStartTime()), 7));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item -> {
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测30天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getThirtyPlantPredictedPowerData_delete(XContext context, GetPlantPredictedPowerDataInput input) {
        //今年（7天总值）/去年（7天的总值）  同比值
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(" IFNULL(avg(power ) , 0 ) power");
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, date);
        queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, 7));
        PlantPredictedPowerDataEnt currentData = mapper.selectOne(queryWrapper);
        queryWrapper.clear();
        queryWrapper.select(" IFNULL(avg(power ) , 0 ) power");
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, -365));
        queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, -358));
        PlantPredictedPowerDataEnt lastData = mapper.selectOne(queryWrapper);
        BigDecimal compare = new BigDecimal(1);
        if (lastData.getPower().compareTo(BigDecimal.ZERO) > 0)
            compare = currentData.getPower().divide(lastData.getPower(), 6, BigDecimal.ROUND_HALF_UP);
        queryWrapper.clear();
        queryWrapper.select("  data_date , power ");
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, -365));
        queryWrapper.lambda().le(PlantPredictedPowerDataEnt::getDataDate, XDateUtils.addDays(date, -335));
        List<PlantPredictedPowerDataEnt> lastList = mapper.selectList(queryWrapper);
        //获取去年30天的数据，预测今年30天的数据
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(lastList, GetPlantPredictedPowerDataOutput.class);
        for (GetPlantPredictedPowerDataOutput item : outputs) {
            item.setPredictedPower(item.getPower().multiply(compare));
            item.setDataDate(XDateUtils.getString(XDateUtils.addDays(XDateUtils.parse(item.getDataDate()), 365)));
        }
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测30天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getThirtyPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerLongTermDataMapper mapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerLongTermDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("id, plant_id, data_date, temperature, humidity, wind_speed, wind_direction, pressure, plane_irradiance, horizontal_irradiance, power as predicted_power, create_time");
        queryWrapper.lambda().eq(PlantPredictedPowerLongTermDataEnt::getPlantId, input.getPlantId());
        queryWrapper.lambda().ge(PlantPredictedPowerLongTermDataEnt::getDataDate, date);
        queryWrapper.lambda().le(PlantPredictedPowerLongTermDataEnt::getDataDate, XDateUtils.addMonth(date, 1));
        List<PlantPredictedPowerLongTermDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        return XListResult.success(outputs);
    }

    @XText("预测电站发电数据")
    @XApiPost
    public XServiceResult doPredictedPowerData(XContext context, DoPlantPredictedPowerDataInput input) {
        doPredictedData(context, input.getPlantId());
        return XServiceResult.OK;
    }

    public void doPredictedData(XContext context, String plantId) {
        try {
            Object ret = HttpUtils.send2("http://127.0.0.1:10098/aiprediction/xgbreason?plantId=" + plantId,
                    "");
            context.getLogger().info(ret.toString());
        } catch (Exception e) {
            throw new RuntimeException("调用训练接口失败");
        }
    }
}