package pps.core.prediction.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.ConfigShowEnt;
import pps.core.prediction.mapper.ConfigShowMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPredictedPowerOutput;
import pps.core.prediction.service.data.plant_predicted_power_ultra_term.GetPlantPredictedPowerUltraTermInput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;


@XService
@XText("电站预测功率4小时数据")
public class PlantPredictedPowerUltraTermService {

    /**
     * 小时功率预测
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerOutput }>
     */
    @XApiGet
    @XText("根据电站预测功率4小时数据")
    public XListResult<GetPredictedPowerOutput> getPlantPredictedPowerUltraTermList(XContext context, GetPlantPredictedPowerUltraTermInput input) {
        DateTime now = DateUtil.date();
        DateTime inputTime;
        String inputStartTime = input.getStartTime();
        if (CharSequenceUtil.isBlank(inputStartTime)) {
            inputTime = DateUtil.beginOfHour(now);
        } else {
            inputTime = DateUtil.parse(now.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + inputStartTime + BusinessConstant.INITIALIZATION_SECOND);
        }
        //检查演示配置
        ConfigShowMapper configMapper = context.getBean(ConfigShowMapper.class);
        ConfigShowEnt configShowEnt = configMapper.selectOne(new LambdaQueryWrapper<ConfigShowEnt>()
                .last("LIMIT 1"));
        //时间出力截至时间
        Date showTime = inputTime;
        int betweenDay = 0;
        DateTime day = inputTime;
        if (ObjectUtil.isAllNotEmpty(configShowEnt, configShowEnt.getIsEnable(), configShowEnt.getChoiceDate())) {
            day = configShowEnt.getIsEnable().equals(0) ? DateUtil.parse(DateUtil.date(configShowEnt.getChoiceDate()).toString(BusinessConstant.DATE_FORMAT_DAY) + " " + inputTime.toString(BusinessConstant.TIME_FORMAT)) : inputTime;
            showTime = DateUtil.parse(day.toString(BusinessConstant.DATE_FORMAT_DAY) + " " + now.toString(BusinessConstant.TIME_FORMAT));
            //计算偏移
            betweenDay = (int) DateUtil.betweenDay(inputTime, day, true);
            if (DateUtil.compare(day, inputTime) > 0) {
                betweenDay = -betweenDay;
            }
        }
        //计算时间范围,开始时间往前推1天，结束时间往后推4小时
        DateTime beginTime = DateUtil.offsetMinute(day, -1441);
        DateTime endTime = DateUtil.offsetHour(day, 4);
        //查光伏预测
        Map<Date, BigDecimal> predictedMap = ServiceUtil.getPredictedPowerMap15(context, 3, input.getPlantId(), beginTime, endTime);
        //查实际功率
        Map<Date, BigDecimal> powerMap = ServiceUtil.getActivePowerMap15(context, input.getPlantId(), beginTime, showTime);
        return XListResult.success(ServiceUtil.getPowerOutput(DateUtil.offsetDay(day, -1), endTime, betweenDay, now, predictedMap, powerMap));
    }
}