package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.base.service.IBaseWellheadCloudService;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.core.auth.HttpRequestClient;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.WellheadDailyProductionSituationEnt;
import pps.core.prediction.excel.listener.ThirdActivePowerDataListener;
import pps.core.prediction.mapper.WellheadDailyProductionSituationMapper;
import pps.core.prediction.service.data.third_active_power.ThirdActivePowerExcelData;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionViewOutput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationInput;
import pps.core.prediction.service.data.wellhead_daily_production_situation.QueryWellheadDailyProductionSituationOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XHttpUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 井口生产情况模块
 *
 * @author ZWT
 * @date 2023/09/27
 */
@XText("井口生产情况模块")
@XService
public class WellheadDailyProductionSituationService {

    /**
     * 井口生产情况--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryWellheadDailyProductionSituationOutput}>
     */
    @XText("井口生产情况--分页列表")
    @XApiPost
    public XPageResult<QueryWellheadDailyProductionSituationOutput> queryWellheadDailyProductionSituationView(XContext context, QueryWellheadDailyProductionSituationInput input) {
        WellheadDailyProductionSituationMapper mapper = context.getBean(WellheadDailyProductionSituationMapper.class);
        PageMethod.startPage(input.getPage(), input.getLimit());
        LambdaQueryWrapper<WellheadDailyProductionSituationEnt> queryWrapper = new LambdaQueryWrapper<WellheadDailyProductionSituationEnt>()
                .eq(WellheadDailyProductionSituationEnt::getWellheadId, input.getWellheadId())
                .eq(WellheadDailyProductionSituationEnt::getWellNumber, input.getWellNumber())
                .orderByDesc(WellheadDailyProductionSituationEnt::getDataDate);
        String executionCycle = input.getExecutionCycle();
        if (CharSequenceUtil.isNotEmpty(executionCycle)) {
            String[] split = CharSequenceUtil.splitToArray(executionCycle, "-");
            queryWrapper.ge(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[0], BusinessConstant.DATE_FORMAT_DAY_C))
                    .le(WellheadDailyProductionSituationEnt::getDataDate, DateUtil.parse(split[1], BusinessConstant.DATE_FORMAT_DAY_C));
        }
        List<WellheadDailyProductionSituationEnt> list = mapper.selectList(queryWrapper);
        PageInfo<WellheadDailyProductionSituationEnt> pageInfo = new PageInfo<>(list);
        List<QueryWellheadDailyProductionSituationOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryWellheadDailyProductionSituationOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 甘特图
     * POST /prediction/wellhead-daily-production-situation/query-rpt-system-start-stop
     * 接口ID：156546795
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-156546795
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetWellheadDailyProductionSituationOutput}>
     */
    @XText("井口生产情况--甘特图")
    @XApiPost
    public XListResult<GetWellheadDailyProductionSituationOutput> queryRptSystemStartStop(XContext context, GetWellheadDailyProductionSituationInput input) {
        String token = ServiceUtil.getToken(context);
        HttpRequestClient client = new HttpRequestClient(token);
        Map<String, Object> param = new HashMap<>(4);
        param.put("Org_Name", input.getStationName());
        param.put("WELL_NAME", input.getWellNumber());
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        url += ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
        String resultStr = client.doPostForm(url, param);
        JSONObject jsonObject = JSON.parseObject(resultStr);
        String data = jsonObject.getString("data");
        List<GetWellheadDailyProductionSituationOutput> outputs = JSON.parseArray(data, GetWellheadDailyProductionSituationOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            outputs = Collections.emptyList();
        }
        return XListResult.success(outputs);
    }

    @XText("井口生产情况--test")
    @XApiGet(anonymous = true)
    public XServiceResult test(XContext context) {
        context.getLogger().info("---------------------------cqToken---------------------------");
        String cqToken = "";
        String api;
        try {
            cqToken = ServiceUtil.getToken(context);
            context.getLogger().info("cqToken: " + cqToken);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
        Map<String, String> header = new HashMap<>(4);
        header.put("token", cqToken);
        HttpRequestClient client = new HttpRequestClient(cqToken);
        context.getLogger().info("---------------------------CQ_WELL_TECH_DAILY---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_TECH_DAILY;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Calc_Date", "2024-03-18");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_WELL_TECH_DAILY: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_STATION_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_STATION_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_RPT_SYSTEM_START_STOP---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_RPT_SYSTEM_START_STOP;
            Map<String, Object> param = new HashMap<>(4);
            param.put("WELL_NAME", "王101-23");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_RPT_SYSTEM_START_STOP: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_WELL_REAL_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_WELL_REAL_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_STATION_DAILY_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_STATION_DAILY_PV_DATA;
            Map<String, Object> param = new HashMap<>(4);
            param.put("Calc_Date", "2024-03-18");
            String s = client.doPostForm(api, param);
            context.getLogger().info("CQ_STATION_DAILY_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------JI_CHENG_TOKEN---------------------------");
        try {
            String t = "http://10.59.249.10:8183/admin-api/system/auth/login";
            Map<String, String> param = new HashMap<>(4);
            param.put("username", "31b5604627e345ebb48175de8f2ee0bdem55ajAwMQ==kj");
            param.put("password", "sdSktfem55ajAwMSFAIzEw8ef354f215d44c0eafd403ab7a1ce74a");
            String result = XHttpUtils.postAsJson(t, JSON.toJSONString(param));
            context.getLogger().info("JI_CHENG_TOKEN: " + JSON.toJSONString(param));
            context.getLogger().info("JI_CHENG_TOKEN: " + result);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_GROUP_REAL_PV_DATA---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_GROUP_REAL_PV_DATA;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_GROUP_REAL_PV_DATA: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        context.getLogger().info("---------------------------CQ_AVG_ACTIVE_POWER---------------------------");
        try {
            api = url + ThirdPartyApiConstant.CQ_AVG_ACTIVE_POWER;
            String s = client.doPostForm(api, new HashMap<>(4));
            context.getLogger().info("CQ_AVG_ACTIVE_POWER: " + s);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return XServiceResult.OK;
    }

    @XText("有功功率历史数据导入")
    @XApiGet(anonymous = true)
    public XServiceResult importHistory(XContext context) {
        String fileName = "D:\\home\\历史数据汇总.xlsx";
        EasyExcel.read(fileName, ThirdActivePowerExcelData.class, new ThirdActivePowerDataListener(context)).sheet().doRead();
        return XServiceResult.OK;
    }

    @XText("井口")
    @XApiGet(anonymous = true)
    public XServiceResult wellTest(XContext context) {
        String wellResult = "[\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"王侧11-8\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 2.11,\n" +
                "\t\t\"starthours\": 5.5,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"2024-05-14T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塞侧29-4\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山31-14\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山33-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山33-14\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山34-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山36-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山37-12\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山39-12\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山40-11\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山41-9\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山43-5\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午249-9\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午252-10\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午252-9\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午291\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午侧249-8\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午平231-04\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山30-15\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山35-12\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山38-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山39-11\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山41-6\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山43-9\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 2.22,\n" +
                "\t\t\"starthours\": 4,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"2024-05-13T16:30:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午250-10\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午251-10\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山37-11\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山33-15\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午222-210X\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山37-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山35-14\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山35-15\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山38-11\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山42-9\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"午246-216\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"山32-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙27-23\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 0.26,\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"停井\",\n" +
                "\t\t\"next_ExecDate\": \"2024-05-14T04:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙30-23\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬64-82\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 3.74,\n" +
                "\t\t\"starthours\": 0.26666666666666669,\n" +
                "\t\t\"status\": \"停井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬342-103\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬50-91\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0.6666666666666666,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬48-93\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 1.03,\n" +
                "\t\t\"starthours\": 0.36666666666666666,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬40-96\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬341-103\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙24-13\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬49-92\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 3.8,\n" +
                "\t\t\"starthours\": 0.36666666666666666,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬42-94\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 2.35,\n" +
                "\t\t\"starthours\": 0.38333333333333338,\n" +
                "\t\t\"status\": \"开井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙侧28-22\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"成62-16\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"reaL_LIQUID_DAILY\": 1.9,\n" +
                "\t\t\"starthours\": 0.26666666666666669,\n" +
                "\t\t\"status\": \"停井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙25-12\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"status\": \"停井\",\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"沙侧30-24\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"黄120\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬平57-17\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t},\n" +
                "\t{\n" +
                "\t\t\"welL_ID\": 0,\n" +
                "\t\t\"welL_NAME\": \"塬47-92\",\n" +
                "\t\t\"caL_DATE\": \"0001-01-01T00:00:00\",\n" +
                "\t\t\"starthours\": 0,\n" +
                "\t\t\"next_ExecDate\": \"0001-01-01T00:00:00\"\n" +
                "\t}\n" +
                "]";
        List<GetThirdCurrentWellConditionViewOutput> wellList = JSON.parseArray(wellResult, GetThirdCurrentWellConditionViewOutput.class);
        IBaseWellheadCloudService service = context.getBean(IBaseWellheadCloudService.class);
        XListResult<DynamicQueryBaseWellheadOutput> result = service.queryBaseWellheadListByParam(context, DynamicQueryBaseWellheadInput.builder()
                .build());
        result.throwIfFail();
        Set<String> collect = result.getResult().stream().map(DynamicQueryBaseWellheadOutput::getWellNumber).collect(Collectors.toSet());
        Set<String> apiCollect = wellList.stream().map(GetThirdCurrentWellConditionViewOutput::getWellNumber).collect(Collectors.toSet());
        System.out.println("-------------------系统里的井口:");
        for (DynamicQueryBaseWellheadOutput output : result.getResult()) {
            System.out.println(output.getWellNumber());
        }
        System.out.println("-------------------长庆接口里的井口:");
        for (GetThirdCurrentWellConditionViewOutput output : wellList) {
            System.out.println(output.getWellNumber());
        }
        System.out.println("-------------------系统有长庆没有的井口:");
        for (String s : collect) {
            if (!apiCollect.contains(s)) {
                System.out.println(s);
            }
        }
        System.out.println("-------------------长庆接口有系统里没有的井口:");
        for (String s : apiCollect) {
            if (!collect.contains(s)) {
                System.out.println(s);
            }
        }
        return XServiceResult.OK;
    }
}